#!/usr/bin/env python3

import unittest
import yaml
import os, fnmatch
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import stretch_body.hello_utils as hu
import stretch_body.version
import unittest
import serial
import fcntl
import glob
from os.path import exists

class Test_SIMPLE_params(unittest.TestCase):
    """
    Check system parameters and identify issues
    """
    test = TestBase('test_SIMPLE_params')


    def test_param_files_exist(self):
        """
        Check that the correct parameter files can be found.
        """
        v=stretch_body.version.__version__
        version_major=int(v.split('.')[0])
        version_minor=int(v.split('.')[1])
        if version_major==0 and version_minor<3:
            ue=exists(hu.get_fleet_directory() + 'stretch_re1_user_params.yaml')
            ce=exists(hu.get_fleet_directory() + 'stretch_re1_factory_params.yaml')
            self.assertTrue(ue,'Legacy parameter file stretch_re1_user_params.yaml missing on HELLO_FLEET_PATH')
            self.assertTrue(ce,'Legacy parameter files stretch_re1_factory_params.yaml missing on HELLO_FLEET_PATH')
        else:
            ue=exists(hu.get_fleet_directory() + 'stretch_user_params.yaml')
            ce=exists(hu.get_fleet_directory() + 'stretch_configuration_params.yaml')
            self.assertTrue(ue,'Parameter format upgrade may be required. For more details, see https://forum.hello-robot.com/t/425')
            self.assertTrue(ce,'Parameter format upgrade may be required. For more details, see https://forum.hello-robot.com/t/425')

    def test_params_log(self):
        """
        Log parameter data from stretch_params.
        """
        #Log stretch params
        fn = self.test.test_result_dir+'/stretch_params_%s.log'%self.test.timestamp
        os.system("stretch_params.py >> %s"%fn)
        print('Logged stretch_params.py to %s' % fn)
        #Copy over yaml files
        param_files_list= glob.glob(hu.get_fleet_directory()+'*.yaml')
        for p in param_files_list:
            fn_no_path=p[p.rfind('/'):]
            new_fn=fn_no_path+'_%s.log'%self.test.timestamp
            os.system("cp %s %s"%(p,self.test.test_result_dir+new_fn))
            print('Logged parameter settings to %s'%new_fn)


test_suite = TestSuite(test=Test_SIMPLE_params.test,failfast=False)
test_suite.addTest(Test_SIMPLE_params('test_param_files_exist'))
test_suite.addTest(Test_SIMPLE_params('test_params_log'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
import unittest
import psutil
import subprocess
import time
import os
import signal
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_suite import TestSuite
from stretch_diagnostics.test_runner import TestRunner


class Test_CPU_usage_temp(unittest.TestCase):
    """
    This stress-tests the CPU and monitors its usage and temperature.
    """

    test = TestBase('test_CPU_usage_temp')

    @classmethod
    def setUpClass(self):
        print('Starting CPU stress test. This test will take 40 seconds.')
        subprocess.Popen("stress -c 4", shell=True)

    @classmethod
    def tearDownClass(self):
        for line in os.popen("ps ax | grep stress | grep -v grep"):
            fields = line.split()
            pid = fields[0]
            os.kill(int(pid), signal.SIGKILL)

    def test_CPU_usage(self):
        """
        Test CPU usage is within acceptable limits
        """
        
        time.sleep(15)
        # Calling psutil.cpu_precent() for 4 seconds
        usage = psutil.cpu_percent(4)
        print('The CPU usage is: {}%'.format(usage))

        self.test.log_data('test_cpu_usage', usage)

        self.assertLess(usage, 55,"CPU usage of %f already high. Kill all running processes before running diagnostics."%usage)

    def test_CPU_temp(self):
        """
        Test CPU temperature is within acceptable limits
        """
        
        time.sleep(20)
        temperature = psutil.sensors_temperatures()['coretemp'][0][1]
        print('The CPU temperature is: ', '{} degC'.format(temperature))

        self.test.log_data('test_cpu_temperature', temperature)
        
        self.assertLess(temperature, 90, 'CPU temp of %f already high. Kill all running processes before running diagnostics.'%temperature)


test_suite = TestSuite(test=Test_CPU_usage_temp.test, failfast=False)

test_suite.addTest(Test_CPU_usage_temp('test_CPU_usage'))
test_suite.addTest(Test_CPU_usage_temp('test_CPU_temp'))

if __name__ == '__main__':
    runner = TestRunner(suite=test_suite, doc_verify_fail=False)
    runner.run()



from colorama import Fore, Style
import os
import glob
import pkg_resources
import click
import pyrealsense2 as rs
import stretch_factory.hello_device_utils as hdu
import stretch_body.scope as scope
import time
import socket
from subprocess import Popen, PIPE, STDOUT
from threading import Thread, Lock
import zipfile
import matplotlib.pyplot as plt
from drawnow import drawnow
from stretch_diagnostics.test_runner import TestRunner

class Dmesg_monitor:
    """
    Run live dmesg fetching in the backgorund using threading. Query the collected dmesg message
    outputs or clear them in between sessions. Save the collected dmesg output at the end.

    TODO: add methods for filtered capturing of Dmesg

    Params
    ------
    print_new_msg =  Prints Dmesg live if True
    log_fn = Optional file path to save the log at stop of dmesg monitor

    """

    def __init__(self, print_new_msg=False, log_fn=None):
        self.prev_out = None
        self.thread = None
        self.output_list = []
        self.is_live = False
        self.lock = Lock()
        self.print_new_msg = print_new_msg
        self.log_fn = log_fn
        os.system("sudo echo ''")

    def dmesg_fetch_clear(self):
        out = Popen("sudo dmesg -c", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE, close_fds=True,
                    stderr=PIPE).stdout.read().decode("utf-8")
        out = str(out).split('\n')
        with self.lock:
            try:
                # Filter ghost lines
                if out[0] != self.prev_out[-1]:
                    for o in out:
                        if len(o) > 0:
                            self.output_list.append(o)
                            if self.print_new_msg:
                                print("[DMESG]...{}".format(o))
            except:
                pass
        self.prev_out = out

    def write_lines_to_file(self, lines, file_path):
        with open(os.path.expanduser(file_path), 'w') as file:
            for line in lines:
                file.write(line + '\n')
        print("DMESG Log saved to: {}".format(self.log_fn))

    def start(self):
        print("Starting DMESG capture....")
        self.is_live = True
        self.thread = Thread(target=self.live)
        self.thread.start()

    def stop(self):
        self.is_live = False
        self.thread.join()
        print("Ending DMESG capture....")
        if self.log_fn is not None:
            self.write_lines_to_file(self.output_list, self.log_fn)

    def live(self):
        while self.is_live:
            self.dmesg_fetch_clear()

    def clear(self):
        with self.lock:
            self.output_list = []

    def get_latest_msg(self):
        return self.prev_out

    def get_output_list(self):
        return self.output_list


class Scope_Sensor_vs_Sensor:
    """
    Scope two sensor values against eachother
    """

    def __init__(self, yrange, title='Scope'):
        plt.ion()  # enable interactivity
        self.fig = plt.figure()
        self.fig.canvas.set_window_title(title)
        plt.ylim(yrange[0], yrange[1])
        self.data_x = []
        self.data_y = []

    def step(self, sensor_value_x, sensor_value_y):
        self.data_x.append(sensor_value_x)
        self.data_y.append(sensor_value_y)
        drawnow(self.make_fig)

    def make_fig(self):
        plt.plot(self.data_x, self.data_y, 'b')

    def savefig(self, filename):
        plt.savefig(filename)


class Scope_Log_Sensor:
    """
    Scope a sensor value and save the figure to a PNG file.
    Optionally provide when callbacks to be called at some time after launch and before exiting
    Return if complete or not
    """

    def __init__(self, duration, y_range=[0, 100.0], title='Sensor', num_points=100, image_fn=None, start_fn=None,
                 start_fn_ts=None, end_fn=None, end_fn_ts=None, delay=0.1):

        self.ts_start = time.time()
        self.duration = duration
        self.data = []
        self.avg = None
        self.delay = delay
        self.image_fn = image_fn
        self.start_fn = start_fn
        self.start_fn_ts = start_fn_ts
        self.end_fn = end_fn
        self.end_fn_ts = end_fn_ts
        self.scope = scope.Scope(num_points=num_points, yrange=y_range, title=title)

    def step(self, sensor_value):
        dt = time.time() - self.ts_start
        if dt < self.duration:
            if self.start_fn_ts is not None and dt > self.start_fn_ts and self.start_fn is not None:
                self.start_fn()
                self.start_fn = None
            if self.end_fn_ts is not None and dt > self.end_fn_ts and self.end_fn is not None:
                self.end_fn()
                self.end_fn = None
            self.data.append(sensor_value)
            self.scope.step_display(sensor_value)
            time.sleep(self.delay)
            return True
        else:
            if self.image_fn is not None:
                print('Saving file %s' % self.image_fn)
                self.scope.savefig(self.image_fn)
                self.image_fn = None
            self.avg = sum(self.data) / len(self.data)
            return False


def get_installed_package_info(find_specific=None):
    packages = {}
    for package in pkg_resources.working_set:
        packages[package.key] = {"version": package.version,
                                 "path": package.location}
    if find_specific:
        return packages[find_specific]
    return packages


def extract_zip(zip_file_path, extract_to_path):
    with zipfile.ZipFile(zip_file_path, 'r') as zip_ref:
        zip_ref.extractall(extract_to_path)


def val_in_range(val_name, val, vmin, vmax, silent=False):
    p = val <= vmax and val >= vmin
    if silent:
        return p
    if p:
        print(Fore.GREEN + '[Pass] ' + val_name + ' = ' + str(val))
        print(Style.RESET_ALL)
        return True
    else:
        print(Fore.RED + '[Fail] ' + val_name + ' = ' + str(val) + ' out of range ' + str(vmin) + ' to ' + str(vmax))
        print(Style.RESET_ALL)
        return False


def val_is_not(val_name, val, vnot):
    if val is not vnot:
        print(Fore.GREEN + '[Pass] ' + val_name + ' = ' + str(val))
        return True
    else:
        print(Fore.RED + '[Fail] ' + val_name + ' = ' + str(val))
        return False


def confirm(question: str) -> bool:
    reply = None
    while reply not in ("y", "n"):
        reply = input(Style.BRIGHT + f"{question} (y/n): " + Style.RESET_ALL).lower()
    return (reply == "y")


def print_instruction(text, ret=0):
    return_text = Fore.BLUE + Style.BRIGHT + 'INSTRUCTION:' + Style.RESET_ALL + Style.BRIGHT + text + Style.RESET_ALL
    if ret == 1:
        return return_text
    else:
        print(return_text)


def print_bright(text):
    print(Style.BRIGHT + Fore.BLUE + text + Style.RESET_ALL)


def print_bright_red(text):
    print(Style.BRIGHT + Fore.RED + text + Style.RESET_ALL)


def system_check_warn(warning=None):
    def decorator(test_item):
        test_item.__system_check_warn__ = True
        test_item.__system_check_warning__ = warning
        return test_item

    return decorator


def command_list_exec(cmd_list):
    cmd = ''
    for c in cmd_list:
        cmd = cmd + c + ';'
    os.system(cmd)


def find_tty_devices():
    devices_dict = {}
    ttyUSB_dev_list = glob.glob('/dev/ttyUSB*')
    ttyACM_dev_list = glob.glob('/dev/ttyACM*')
    for d in ttyACM_dev_list:
        devices_dict[d] = {"serial": extract_udevadm_info(d, 'ID_SERIAL_SHORT'),
                           "vendor": extract_udevadm_info(d, 'ID_VENDOR'),
                           "model": extract_udevadm_info(d, 'ID_MODEL'),
                           "path": extract_udevadm_info(d, 'DEVPATH')}
    for d in ttyUSB_dev_list:
        devices_dict[d] = {"serial": extract_udevadm_info(d, 'ID_SERIAL_SHORT'),
                           "vendor": extract_udevadm_info(d, 'ID_VENDOR'),
                           "model": extract_udevadm_info(d, 'ID_MODEL'),
                           "path": extract_udevadm_info(d, 'DEVPATH')}
    return devices_dict


def get_serial_nos_from_udev(udev_file_full_path, device_name):
    sns = []
    try:
        f = open(udev_file_full_path, 'r')
        x = f.readlines()
        f.close()
        lines = []
        for xx in x:
            if xx.find(device_name) > 0 and xx[0] != '#':
                lines.append(xx)
        for l in lines:
            ll = l.split(',')
            for q in ll:
                if q.find('serial') > -1:
                    s = q[q.find('"') + 1:q.rfind('"')]
                    if len(s) == 8 or len(s) == 32:  # FTDI or Arduino
                        sns.append(s)
    except:
        pass
    return sns


def find_ftdi_devices_sn():
    devices_dict = {}
    ttyUSB_dev_list = glob.glob('/dev/ttyUSB*')
    for d in ttyUSB_dev_list:
        devices_dict[d] = extract_udevadm_info(d, 'ID_SERIAL_SHORT')
    return devices_dict


def find_arduino_devices_sn():
    devices_dict = {}
    ttyACM_dev_list = glob.glob('/dev/ttyACM*')
    for d in ttyACM_dev_list:
        devices_dict[d] = extract_udevadm_info(d, 'ID_SERIAL_SHORT')
    return devices_dict


def extract_udevadm_info(usb_port, ID_NAME=None):
    """
    Extracts usb device attributes with the given attribute ID_NAME

    example ID_NAME:
    ID_SERIAL_SHORT
    ID_MODEL
    DEVPATH
    ID_VENDOR_FROM_DATABASE
    ID_VENDOR
    """
    value = None
    dname = bytes(usb_port[5:], 'utf-8')
    out = hdu.exec_process([b'udevadm', b'info', b'-n', dname], True)
    if ID_NAME is None:
        value = out.decode(encoding='UTF-8')
    else:
        for a in out.split(b'\n'):
            a = a.decode(encoding='UTF-8')
            if "{}=".format(ID_NAME) in a:
                value = a.split('=')[-1]
    return value


def get_rs_details():
    """
    Returns the details of the first found realsense devices in the bus
    """
    dev = None
    ctx = rs.context()
    devices = ctx.query_devices()
    found_dev = False
    for dev in devices:
        if dev.get_info(rs.camera_info.serial_number):
            found_dev = True
            break
    if not found_dev:
        print('No RealSense device found.')
        return None

    data = {}
    data["device_pid"] = dev.get_info(rs.camera_info.product_id)
    data["device_name"] = dev.get_info(rs.camera_info.name)
    data["serial"] = dev.get_info(rs.camera_info.serial_number)
    data["firmware_version"] = dev.get_info(rs.camera_info.firmware_version)

    return data


def check_internet():
    try:
        socket.create_connection(("www.github.com", 80))
        return True
    except OSError:
        pass
    return False


def center_string(text, length=75, ch=' '):
    prefix_n = ((length - len(text)) // 2) - 1
    suffix_n = (length - len(text) - prefix_n) - 1
    return f"{ch * prefix_n} {text} {ch * suffix_n}"


def run_gist(gist_id):
    cmd = f"wget https://gist.githubusercontent.com/{gist_id}/raw/ --no-check-certificate"
    os.system(cmd)
    os.system("mv index.html /tmp/misc_test.py")
    import sys
    sys.path.append('/tmp')
    import misc_test
    runner=TestRunner(misc_test.test_suite)
    runner.run()
    if runner.test_result_filename is not None:
        print(Fore.BLUE + 'GIST test complete.\n Results: %s \nSend results to support@hello-robot.com'%runner.test_result_filename)
    else:
        print(Fore.YELLOW + 'GIST test failed to generate results data')
    os.system("rm /tmp/misc_test.py")



import unittest
from colorama import Fore, Style

class TestResult(unittest.runner.TextTestResult):
    def startTest(self, test):
        super(TestResult, self).startTest(test)
        print('\n')
        test_id = test.id().split('.')[-1]
        print(test_id)
        print('-' * len(test_id))
        if test.shortDescription():
            print('Description:\n' + test.shortDescription() + '\n')
        else:
            print(Fore.YELLOW + "[WARNING]: Description Missing for test: {}".format(test_id) + Style.RESET_ALL)

    def stopTest(self, test):
        super(TestResult, self).stopTest(test)
        result = test.defaultTestResult()
        test._feedErrorsToResult(result, test._outcome.errors)
        ok = result.wasSuccessful()
        errors = result.errors
        failures = result.failures

        if ok:
            print(Fore.GREEN)
            print('Test Case passed')
            print(Style.RESET_ALL)
        else:
            print(Fore.RED)
            print('Test Case Failed')
            print('{} errors and {} failures'.format(len(errors), len(failures)))
            print(Style.RESET_ALL)



#!/usr/bin/env python3
import stretch_diagnostics.test_helpers as test_helpers
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.stepper
import stretch_body.hello_utils as hu
import stretch_factory.firmware_updater as firmware_updater
import sys
class Test_SIMPLE_firmware(unittest.TestCase):
    """
    Test firmware versions of PCBAs
    """
    test = TestBase('test_SIMPLE_firmware')


    def test_firmware_log_versions(self):
        """
            Log versions of installed firmware
        """
        use_device = {'hello-motor-arm': True, 'hello-motor-right-wheel': True, 'hello-motor-left-wheel': True,
                      'hello-pimu': True, 'hello-wacc': True, 'hello-motor-lift': True}
        r = firmware_updater.RecommendedFirmware(use_device)
        # Log recommendations
        old_stdout = sys.stdout
        log_fn = self.test.test_result_dir + '/firmware_versions_%s.log' % self.test.timestamp
        print('Logging current firmware version to %s' % log_fn)
        log_file = open(log_fn, "w")
        sys.stdout = log_file
        r.pretty_print()
        sys.stdout = old_stdout
        log_file.close()
    def test_firmware_version(self):
        """
            Check if at recommended firmware versions
        """
        use_device = {'hello-motor-arm': True, 'hello-motor-right-wheel': True, 'hello-motor-left-wheel': True,
                      'hello-pimu': True, 'hello-wacc': True, 'hello-motor-lift': True}
        r = firmware_updater.RecommendedFirmware(use_device)
        #Check for issues
        all_valid=True
        all_up_to_date=True
        for device_name in r.recommended.keys():
            v=r.fw_installed.is_device_valid(device_name)
            if not v:
                all_valid=False
                self.assertTrue(0,'Firmware device not valid for device %s'%device_name)

            version = r.fw_installed.get_version(device_name)
            if r.recommended[device_name]!=None:
                if not r.recommended[device_name] == version:
                    all_up_to_date=False
                    m='Firmware for %s not at latest version. See REx_firmware_updater.py'%device_name
                    self.assertTrue(0,m)

        self.assertTrue(all_valid,msg='Not all devices on bus')
        self.assertTrue(all_up_to_date, 'Firmware not up to date on all devices')

test_suite = TestSuite(test=Test_SIMPLE_firmware.test,failfast=False)
test_suite.addTest(Test_SIMPLE_firmware('test_firmware_log_versions'))
test_suite.addTest(Test_SIMPLE_firmware('test_firmware_version'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import Scope_Log_Sensor
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.dynamixel_hello_XL430
import stretch_body.dynamixel_XL430
import stretch_body.stretch_gripper
import time

class Test_GRIPPER_broken_string(unittest.TestCase):
    """
    Test Gripper for mechanical issues
    """
    test = TestBase('test_GRIPPER_broken_string')

    def test_broken_string(self):
        """
        Check range of servo motion for broken string
        """
        servo = stretch_body.stretch_gripper.StretchGripper()
        self.assertTrue(servo.startup(), msg='Not able to connect to servo stretch_gripper')
        self.assertTrue(servo.do_ping(), msg='Not able to ping servo stretch_gripper')
        servo.pull_status()
        x0=servo.status['pos']
        print('Starting at position %f (rad)'%x0)
        servo.enable_pwm()
        servo.set_pwm(servo.params['pwm_homing'][0])
        ts = time.time()
        time.sleep(1.0)
        timeout = False
        pos_log=[x0]

        while servo.motor.is_moving() and not timeout:
            #print(servo.motor.is_moving())
            timeout = time.time() - ts > 10.0
            time.sleep(0.1)
            servo.pull_status()
            pos_log.append(servo.status['pos'])
            print("Pos: %f"%servo.status['pos'])
        servo.stop()
        print('Test stopped at position %f (rad)'%pos_log[-1])
        print('Motion stopped in %f seconds'%(time.time()-ts))
        servo.set_pwm(0)
        if timeout:
            self.test.add_hint('Gripper servo motion did not stop during homing. Possible broken string')
        self.assertFalse(timeout,msg='Gripper servo motion did not stop during homing. Possible broken string')
        self.test.log_data('stretch_gripper_pos_log', pos_log)

test_suite = TestSuite(test=Test_GRIPPER_broken_string.test,failfast=False)
test_suite.addTest(Test_GRIPPER_broken_string('test_broken_string'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3

import unittest
import yaml
import os, fnmatch
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest

class Test_SIMPLE_usb_devices_on_bus(unittest.TestCase):
    """
    Test USB Devices on Bus
    """
    test = TestBase('test_SIMPLE_usb_devices_on_bus')

    def test_ttyACMx_devices_on_bus(self):
        """
        Verify if all the hello-* usb devices are present in the bus
        """
        ttyACM_devices = {'/dev/ttyACM0': False,
                         '/dev/ttyACM1': False,
                         '/dev/ttyACM2': False,
                         '/dev/ttyACM3': False,
                         '/dev/ttyACM4': False,
                         '/dev/ttyACM5': False}

        listOfFiles = os.listdir('/dev')
        pattern = "hello*"
        for entry in listOfFiles:
            if fnmatch.fnmatch(entry, pattern):
                ttyACM_devices[entry] = True

        for k in ttyACM_devices:
            with self.subTest(msg=k):
                self.assertTrue(ttyACM_devices[k], msg='{} Not found'.format(k))
        print(yaml.dump(ttyACM_devices))
        self.test.log_data('devices_on_usb',ttyACM_devices)
    def test_usb_devices_on_bus(self):
        """
        Verify if all the hello-* usb devices are present in the bus
        """
        robot_devices = {'hello-wacc': False,
                         'hello-motor-left-wheel': False,
                         'hello-pimu': False,
                         'hello-lrf': False,
                         'hello-dynamixel-head': False,
                         'hello-dynamixel-wrist': False,
                         'hello-motor-arm': False,
                         'hello-motor-right-wheel': False,
                         'hello-motor-lift': False,
                         'hello-respeaker': False}

        listOfFiles = os.listdir('/dev')
        pattern = "hello*"
        for entry in listOfFiles:
            if fnmatch.fnmatch(entry, pattern):
                robot_devices[entry] = True

        for k in robot_devices:
            with self.subTest(msg=k):
                self.assertTrue(robot_devices[k], msg='{} Not found'.format(k))
        print(yaml.dump(robot_devices))
        self.test.log_data('devices_on_usb',robot_devices)

test_suite = TestSuite(test=Test_SIMPLE_usb_devices_on_bus.test,failfast=False)
test_suite.addTest(Test_SIMPLE_usb_devices_on_bus('test_usb_devices_on_bus'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
import stretch_diagnostics.test_helpers as test_helpers
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.stepper
import stretch_body.hello_utils as hu
import os
import  click
import time
import stretch_body.pimu as pimu

class Test_STEPPER_sync(unittest.TestCase):
    """
    Test Stepper calibration data consistency
    """
    test = TestBase('test_STEPPER_sync')

    def stepper_sync(self,joint,xd,error_thresh):
        """
        Check that stepper motion halts on runstop
        """
        p=pimu.Pimu()
        self.assertTrue(p.startup())
        p.runstop_event_reset()
        p.push_command()

        log={'sync_x0':0,'sync_x1':0,'sync_x2':0}
        print('Testing sync on stepper %s'%joint)
        motor=stretch_body.stepper.Stepper('/dev/'+joint)
        self.assertTrue(motor.startup())

        print('Enabling sync and checking for small motion...')
        motor.pull_status()
        log['sync_x0']=motor.status['pos']
        motor.enable_sync_mode()
        motor.push_command()

        motor.set_command(motor.MODE_POS_TRAJ_INCR,x_des=xd)
        motor.push_command()
        time.sleep(1.5)

        motor.pull_status()
        log['sync_x1'] = motor.status['pos']

        p.trigger_motor_sync()
        time.sleep(1.5)

        motor.pull_status()
        log['sync_x2'] = motor.status['pos']

        self.test.log_data('test_sync_' + joint, log)

        #Check that no motion before pimu trigger
        error = abs(log['sync_x0'] - log['sync_x1'])
        msg = '%s: Waiting on sync and motion of %f (rad) relative to expected of %f' % (joint, error, 0)
        print(msg)
        self.assertTrue(error < error_thresh, msg=msg)  # Within error_thresh radians

        # Check that motion after pimu trigger
        error = abs(abs(log['sync_x1'] - log['sync_x2'])-xd)
        msg = '%s: Triggered sync and motion of %f (rad) relative to expected of %f' % (joint,abs(log['sync_x1'] - log['sync_x2']), xd)
        print(msg)
        self.assertTrue(error < error_thresh, msg=msg)  # Within error_thresh radians
        motor.stop()

    def test_sync_right_wheel(self):
        "Check that sync is working for the right wheel"
        self.stepper_sync('hello-motor-right-wheel',xd=1.0,error_thresh=0.2)

    def test_sync_left_wheel(self):
        "Check that sync is working for the left wheel"
        self.stepper_sync('hello-motor-left-wheel',xd=1.0,error_thresh=0.2)

    def test_sync_arm(self):
        "Check that sync is working for the arm"
        click.secho('Ensure ARM is not near its hardstop. Hit enter when ready', fg="yellow")
        input()
        self.stepper_sync('hello-motor-arm',xd=0.5,error_thresh=0.1)

    def test_sync_lift(self):
        "Check that sync is working for the lift"
        click.secho('Ensure LIFT is not near its hardstop. Hit enter when ready', fg="yellow")
        input()
        self.stepper_sync('hello-motor-lift',xd=0.5,error_thresh=0.2)

test_suite = TestSuite(test=Test_STEPPER_sync.test,failfast=False)
test_suite.addTest(Test_STEPPER_sync('test_sync_right_wheel'))
test_suite.addTest(Test_STEPPER_sync('test_sync_left_wheel'))
test_suite.addTest(Test_STEPPER_sync('test_sync_arm'))
test_suite.addTest(Test_STEPPER_sync('test_sync_lift'))
if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import Scope_Sensor_vs_Sensor
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.lift
import stretch_body.hello_utils as hu
import time
import click

class test_LIFT_effort_through_range_of_motion(unittest.TestCase):
    """
    Check if lift can move through its range-of-motion using nominal currents
    """
    test = TestBase('test_LIFT_effort_through_range_of_motion')

    def test_effort_through_range_of_motion(self):
        """
        Check if arm can move through its range-of-motion using nominal currents
        """
        data = {}
        l = stretch_body.lift.Lift()
        self.assertTrue(l.startup(),'Unable to startup lift')
        #Move through manually set mechanical limits without any safty protections on
        #Log what the worst case motor effort is required
        #Also don't require homing to be done as that may be broken

        l.motor.enable_safety()
        l.motor.disable_sync_mode()
        l.motor.disable_guarded_mode()
        l.push_command()

        #First collect physical hardstop postions
        click.secho('Manually move lift to fully lowered position', fg="yellow")
        click.secho('Hit enter when ready', fg="yellow")
        input()
        l.pull_status()
        data['x_lower_manual']=l.status['pos']
        xr=l.motor.status['pos']

        click.secho('Manually move lift to fully raised position', fg="yellow")
        click.secho('Hit enter when ready', fg="yellow")
        input()
        l.pull_status()
        data['x_raise_manual'] = l.status['pos']
        xe=l.motor.status['pos']

        l.motor.set_motion_limits(xr,xe)
        l.push_command()

        #Check if physical range of motion reasonable
        data['range_of_motion_m']=abs(data['x_raise_manual']-data['x_lower_manual'])
        lim0=l.params['calibration_range_bounds'][0]-.005
        lim1 = l.params['calibration_range_bounds'][1] + .005
        in_bounds = data['range_of_motion_m']>lim0 and data['range_of_motion_m']<lim1
        msg_in_bounds = 'Manual range of motion of %f compared to expected of %f to %f'%(data['range_of_motion_m'],lim0,lim1)
        print(msg_in_bounds)

        eff_min=0
        eff_max=0
        # Move in first direction and collect pos vs effort
        sls = Scope_Sensor_vs_Sensor(yrange=[-100.0, 100], title='Lower LIft Effort (pct) vs Position (m) ')
        #Move to fully lowered (from motor API so don't require homing to be working)
        l.motor.set_command(mode=l.motor.MODE_POS_TRAJ,x_des=xr)
        l.push_command()
        ts=time.time()
        while time.time()-ts<12.0:
            l.pull_status()
            sls.step(l.status['pos']-l.motor_rad_to_translate_m(xr),l.motor.status['effort_pct'])
            eff_min=min(eff_min,l.motor.status['effort_pct'])
            eff_max=max(eff_max,l.motor.status['effort_pct'])
            #print('Pos %f | Effort %f'%(l.status['pos'],l.motor.status['effort_pct']))
            time.sleep(0.01)
        data['log_lower_pos']=sls.data_x
        data['log_lower_effort'] = sls.data_y
        image_fn = self.test.test_result_dir + '/lift_lower_effort_vs_pos_%s.png' % self.test.timestamp
        sls.savefig(image_fn)

        # Move in second direction and collect pos vs effort
        sls2 = Scope_Sensor_vs_Sensor(yrange=[-100.0, 100], title='Raise Lift Effort (pct) vs Position (m) ')
        # Move to fully raiseed (from motor API so don't require homing to be working)
        l.motor.set_command(mode=l.motor.MODE_POS_TRAJ, x_des=xe)
        l.push_command()
        ts = time.time()
        while time.time() - ts < 12.0:
            l.pull_status()
            sls2.step(l.status['pos']-l.motor_rad_to_translate_m(xr), l.motor.status['effort_pct'])
            eff_min=min(eff_min,l.motor.status['effort_pct'])
            eff_max=max(eff_max,l.motor.status['effort_pct'])
            #print('Pos %f | Effort %f' % (l.status['pos'], l.motor.status['effort_pct']))
            time.sleep(0.01)
        data['log_raise_pos'] = sls2.data_x
        data['log_raise_effort'] = sls2.data_y
        image_fn = self.test.test_result_dir + '/lift_raise_effort_vs_pos_%s.png' % self.test.timestamp
        sls2.savefig(image_fn)

        data['min_effort_pct']=eff_min
        data['max_effort_pct'] = eff_max
        print('Min effort of: %f . Max effort of %f'%(eff_min,eff_max))
        data['contact_thresh_max']=l.params['contact_models']['effort_pct']['contact_thresh_max']
        data['contact_thresh_homing'] = l.params['contact_models']['effort_pct']['contact_thresh_homing']
        print('Guarded contact efforts of %f to %f'%(l.params['contact_models']['effort_pct']['contact_thresh_max'][0],l.params['contact_models']['effort_pct']['contact_thresh_max'][1]))
        print('Homing guarded contact efforts of %f to %f' % (l.params['contact_models']['effort_pct']['contact_thresh_homing'][0],l.params['contact_models']['effort_pct']['contact_thresh_homing'][1]))
        click.secho('Data collection done.', fg="yellow")
        click.secho('Hit enter to finish', fg="yellow")
        input()

        self.test.log_data('test_effort_through_range_of_motion', data)

        l.stop()
        self.assertTrue(in_bounds, msg_in_bounds)




test_suite = TestSuite(test=test_LIFT_effort_through_range_of_motion.test,failfast=False)
test_suite.addTest(test_LIFT_effort_through_range_of_motion('test_effort_through_range_of_motion'))


if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
import stretch_diagnostics.test_helpers as test_helpers
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.stepper
import stretch_body.hello_utils as hu
import os
import  click
import time

class Test_STEPPER_calibration_data_match(unittest.TestCase):
    """
    Test Stepper calibration data consistency
    """
    test = TestBase('test_STEPPER_calibration_data_match')

    def stepper_calibration_data_match(self,s):
        """
        Check that encoder calibration YAML matches what's in flash
        """
        m = stretch_body.stepper.Stepper(usb='/dev/' + s)
        self.assertTrue(m.startup(), msg='Not able to startup stepper %s' % s)
        print('Comparing flash data and encoder data for %s. This will take a minute...' % s)
        log={}
        log['yaml_data']=m.read_encoder_calibration_from_YAML()
        log['flash_data']=m.read_encoder_calibration_from_flash()
        #time.sleep(1.0)
        #m.turn_rpc_interface_on()
        #m.stop()
        log['match']=(log['yaml_data'] == log['flash_data'])
        self.check_if_calibration_corrupted(log['flash_data'])
        self.assertTrue(log['match'],'Encoder calibration in flash for %s does not match that in YAML. See REx_stepper_calibration_flash_to_YAML.py' % s)
        self.test.log_data('encoder_calibration_%s'%s, log)

    def check_if_calibration_corrupted(self, data):
        """
        Function to check if the collected data is corrupt
        """
        Target_data_len = 16384
        self.test.log_params('target_data_len', Target_data_len)
        self.assertEqual(len(data), Target_data_len, msg='Incorrect encoder data length.')

        cnt = 0
        for d in data:
            if d == float(0):
                cnt = cnt + 1
        self.assertGreaterEqual(Target_data_len / 2,cnt, msg='Invalid Encoder Data. Non-zero data expected.')

    def test_calibration_data_match_lift(self):
        """
        Check that LIFT encoder calibration YAML matches what's in flash
        """
        print()
        click.secho('Lift may drop. Place clamp under lift. Hit enter when ready', fg="yellow")
        input()
        self.stepper_calibration_data_match('hello-motor-lift')

    def test_calibration_data_match_arm(self):
        """
        Check that ARM encoder calibration YAML matches what's in flash
        """
        self.stepper_calibration_data_match('hello-motor-arm')

    def test_calibration_data_match_left_wheel(self):
        """
        Check that LEFT WHEEL encoder calibration YAML matches what's in flash
        """
        self.stepper_calibration_data_match('hello-motor-left-wheel')

    def test_calibration_data_match_right_wheel(self):
        """
        Check that RIGHT WHEEL encoder calibration YAML matches what's in flash
        """
        self.stepper_calibration_data_match('hello-motor-right-wheel')

    def test_stepper_startup_after_flash_read(self):
        for s in self.steppers:
            m = stretch_body.stepper.Stepper(usb='/dev/' + s)
            self.assertTrue(m.startup(), msg='Not able to startup stepper %s' % s)
            #m.stop()


test_suite = TestSuite(test=Test_STEPPER_calibration_data_match.test,failfast=False)
test_suite.addTest(Test_STEPPER_calibration_data_match('test_calibration_data_match_lift'))
test_suite.addTest(Test_STEPPER_calibration_data_match('test_calibration_data_match_arm'))
test_suite.addTest(Test_STEPPER_calibration_data_match('test_calibration_data_match_right_wheel'))
test_suite.addTest(Test_STEPPER_calibration_data_match('test_calibration_data_match_left_wheel'))
#test_suite.addTest(Test_STEPPER_calibration_data_match('test_stepper_startup_after_flash_read'))
if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
import unittest
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_suite import TestSuite
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_helpers import Dmesg_monitor, get_rs_details
from colorama import Fore, Style
from subprocess import Popen, PIPE, STDOUT
import time
import numpy as np
import click
import stretch_body.robot
import subprocess

# Include all the known kernel non problematic messages here
# index     1:no.occured 2:no.Acceptable_Occurances 3:Message
known_msgs = [[0, 30, 'uvcvideo: Failed to query (GET_CUR) UVC control'],
              [0, 15, 'Non-zero status (-71) in video completion handler'],
              [0, 15, 'No report with id 0xffffffff found'],
              [0, 10, 'uvcvideo: Found UVC 1.50 device Intel(R) RealSense(TM) Depth Camera 435'],
              [0, 5, 'uvcvideo: Unable to create debugfs'],
              [0, 4, 'hid-sensor-hub'],
              [0, 6, 'input: Intel(R) RealSense(TM) Depth Ca'],
              [0, 1, 'uvcvideo: Failed to resubmit video URB (-1).'],
              [0, 1, 'Netfilter messages via NETLINK v0.30.'],
              [0, 1, 'USB disconnect']]


def scan_head_sequence(robot):
    """
    Head Pan Tilt Sequence for 30s
    """
    robot.head.home()
    time.sleep(1)

    n = 15
    delay = 0.1
    tilt_moves = np.linspace(-1.57, 0, n)
    pan_moves = np.linspace(1.57, -3.14, n)

    for j in range(0, 3):
        for i in range(n):
            robot.head.move_to('head_tilt', tilt_moves[i])
            robot.head.move_to('head_pan', pan_moves[i])
            time.sleep(delay)
        time.sleep(0.8)

        for i in range(n):
            robot.head.move_to('head_tilt', np.flip(tilt_moves)[i])
            robot.head.move_to('head_pan', np.flip(pan_moves)[i])
            time.sleep(delay)
        time.sleep(0.8)

        for i in range(n):
            robot.head.move_to('head_tilt', np.flip(tilt_moves)[i])
            robot.head.move_to('head_pan', pan_moves[i])
            time.sleep(delay)
        time.sleep(0.8)

        for i in range(n):
            robot.head.move_to('head_tilt', tilt_moves[i])
            robot.head.move_to('head_pan', np.flip(pan_moves)[i])
            time.sleep(delay)
        time.sleep(0.8)

    robot.head.home()


class Test_REALSENSE_cable(unittest.TestCase):
    """
    Testing the functioning of the Realsense cable
    """

    # test object is always expected within a TestCase Class
    test = TestBase('test_REALSENSE_cable')


    @classmethod
    def setUpClass(self):
        dmesg_log_fn = "{}/{}_{}.log".format(self.test.results_directory_test_specific,
                                             "dmesg",
                                             self.test.timestamp)
        self.dmesg = Dmesg_monitor(print_new_msg=True, log_fn=dmesg_log_fn)
        self.dmesg.start()

    @classmethod
    def tearDownClass(self):
        self.dmesg.stop()
        print("\nCollected DMESG")
        print("---------------")
        for l in self.dmesg.output_list:
            print(l)

    def test_USB3_2_connection(self):
        """
        Check that Realsense camera is on USB3.2 connection
        """
        out = Popen("rs-enumerate-devices| grep Usb | grep 3.2", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                    close_fds=True).stdout.read()

        if len(out):
            print('Confirmed USB 3.2 connection to realsense device')

        self.assertIsNot(len(out), 0, msg='Did not find USB 3.2 connection to realsense device')

    def test_realsense_on_usb_bus(self):
        """
        Check that Realsense camera is on USB bus.
        """
        print('---- Checking for Intel D435i ----')
        cmd = "lsusb -d 8086:0b3a"
        returned_value = subprocess.call(cmd, shell=True)  # returns the exit code in unix
        self.assertEqual(returned_value, 0,'Realsense D435i not found at USB Bus')

    def test_realsense_details(self):
        """
        Capture realsense details and log
        """
        d = get_rs_details()
        self.assertIsNotNone(d,'Not able to launch Realsense driver. It may be conflicting with ROS')
        self.test.log_data('realsense_details', d)

    def check_dmesg(self, msgs):
        unknown_msgs = []
        excessive_msgs = []
        unexpected_msgs = []
        no_error = True
        for m in msgs:
            if len(m):
                found = False
                for i in range(len(known_msgs)):
                    if m.find(known_msgs[i][2]) != -1:
                        found = True
                        known_msgs[i][0] = known_msgs[i][0] + 1
                if not found:
                    unknown_msgs.append(m)
        for i in range(len(known_msgs)):
            if known_msgs[i][0] >= known_msgs[i][1]:

                print(Fore.YELLOW + '[Warning] Excessive dmesg warnings (%d) of: %s' % (
                known_msgs[i][0], known_msgs[i][2]))
                excessive_msgs.append(known_msgs[i][2])
                no_error = False
        if len(unknown_msgs):
            print('[Warning] Unexpected dmesg warnings (%d)' % len(unknown_msgs))
            unexpected_msgs = unknown_msgs
            no_error = False
            for i in unknown_msgs:
                print(i)
        if no_error:
            print(Fore.GREEN + '[Pass] No unexpected dmesg warnings')
        print(Style.RESET_ALL)
        self.test.log_data("excessive_dmesgs", excessive_msgs)
        self.test.log_data("unexpected_dmesgs", unexpected_msgs)
        self.assertTrue(no_error, "Errors captured in DMESGs.")

    def test_monitor_dmesg_with_head_motion(self):
        """
        Move head pan-tilt continuously for 30s and observe USB messages in DMESG
        """
        robot = stretch_body.robot.Robot()
        input(click.style(
            "Make sure the head is free to move without obstructions. The head will spin for 2 minutes. Press ENTER",
            fg="yellow", bold=True))
        print("Started Head rotation....")
        self.assertTrue(robot.startup(), "Unable to startup the robot")
        scan_head_sequence(robot)
        scan_head_sequence(robot)
        scan_head_sequence(robot)
        scan_head_sequence(robot)
        robot.stop()
        print("Stoped Head rotation....")
        self.check_dmesg(self.dmesg.output_list)


test_suite = TestSuite(test=Test_REALSENSE_cable.test, failfast=False)
test_suite.addTest(Test_REALSENSE_cable('test_USB3_2_connection'))
test_suite.addTest(Test_REALSENSE_cable('test_realsense_on_usb_bus'))
test_suite.addTest(Test_REALSENSE_cable('test_realsense_details'))
test_suite.addTest(Test_REALSENSE_cable('test_monitor_dmesg_with_head_motion'))

if __name__ == '__main__':
    runner = TestRunner(suite=test_suite, doc_verify_fail=False)
    runner.run()



#!/usr/bin/env python3
import stretch_diagnostics.test_helpers as test_helpers
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.stepper
import stretch_body.hello_utils as hu
import os
import  click

class Test_SIMPLE_steppers(unittest.TestCase):
    """
    Test Stepper basic configuration
    """
    test = TestBase('test_SIMPLE_steppers')
    steppers = ['hello-motor-right-wheel','hello-motor-left-wheel','hello-motor-arm','hello-motor-lift']

    def test_steppers_present(self):
        """
        Check that steppers are present
        """
        for s in self.steppers:
            self.assertTrue(hdu.is_device_present('/dev/'+s),msg='Stepper %s not found on USB bus'%s)
    
    def test_steppers_calibrated(self):
        """
        Check that steppers have encoder calibration
        """
        all_calibrated=True
        calibrated_log={}
        for s in self.steppers:
            m=stretch_body.stepper.Stepper(usb='/dev/'+s)
            self.assertTrue(m.startup(),msg='Not able to startup stepper %s'%s)
            m.pull_status()
            calibrated=m.status['calibration_rcvd']
            calibrated_log[s]=calibrated
            self.assertTrue(calibrated,'Encoder calibration not in flash for %s'%s)
            m.stop()
        self.test.log_data('encoder_calibrated',calibrated_log)


    def test_stepper_calibration_files(self):
        """
        Check that encoder calibration is present in stretch_user and etc (and they match)
        """
        calibration_data_log = {}
        for s in self.steppers:
            m = stretch_body.stepper.Stepper(usb='/dev/' + s)
            fn_encoder_calibration = m.name+'_'+m.params['serial_no']+'.yaml'
            fn_user=hu.get_fleet_directory()+'calibration_steppers/'+fn_encoder_calibration
            fn_etc ='/etc/hello-robot/'+hu.get_fleet_id()+'/calibration_steppers/'+fn_encoder_calibration
            etc_exists=os.path.exists(fn_etc)
            user_exists=os.path.exists(fn_user)
            data_match=False
            if etc_exists and user_exists:
                e=hu.enc_data=hu.read_fleet_yaml(fn_etc)
                u = hu.enc_data = hu.read_fleet_yaml(fn_user)
                data_match=(e==u)
            calibration_data_log[s]={'etc_file_exists':etc_exists,'user_file_exists':user_exists,'file_data_match':data_match}
            self.assertTrue(etc_exists,'Encoder calibration file missing: %s'%fn_etc)
            self.assertTrue(user_exists,'Encoder calibration file missing: %s'%fn_user)
            self.assertTrue(data_match,'Encoder calibration data files do not match: %s and %s' % (fn_etc,fn_user))

            #m.stop()
        self.test.log_data('encoder_calibration_files', calibration_data_log)


test_suite = TestSuite(test=Test_SIMPLE_steppers.test,failfast=False)
test_suite.addTest(Test_SIMPLE_steppers('test_steppers_present'))
test_suite.addTest(Test_SIMPLE_steppers('test_steppers_calibrated'))
test_suite.addTest(Test_SIMPLE_steppers('test_stepper_calibration_files'))


if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()




"""
This Library consists of all the test_* test suite names listed and ordered.
The dictionary "test_order" should have all the robot batch specific test lists.
"""

test_order= {
    'simple':[
        'test_SIMPLE_wifi',
        'test_SIMPLE_params',
        'test_SIMPLE_stretch_body_background',
        'test_SIMPLE_usb_devices_on_bus',
        'test_SIMPLE_udev',
        'test_SIMPLE_realsense_status',
        'test_SIMPLE_pimu',
        'test_SIMPLE_wacc',
        'test_SIMPLE_rplidar',
        'test_SIMPLE_steppers',
        'test_SIMPLE_firmware',
        'test_SIMPLE_dynamixel_configure',
        'test_SIMPLE_accessories',
        'test_SIMPLE_software_packages',
        'test_SIMPLE_filesystem',
        'test_SIMPLE_capture_system'
        ],
    'power':[
        'test_POWER_charger',
        'test_POWER_battery_loading'
        ],
    'cpu':['test_CPU_usage_temp'],
    'ros':[
        'test_ROS_sourced_distro',
        'test_ROS_calibration',
        'test_ROS_urdf'
        ],
    'dynamixel':[
        'test_DYNAMIXEL_measure_efforts',
        'test_DYNAMIXEL_measure_range_of_motion',
        'test_DYNAMIXEL_check_zero_position'
        ],
    'gripper':[
        'test_GRIPPER_broken_string',
        ],
     'stepper':[
         'test_STEPPER_calibration_data_match',
         'test_STEPPER_runstop',
         'test_STEPPER_sync'
         ],
    'arm':['test_ARM_effort_through_range_of_motion'],
    'lift': ['test_LIFT_effort_through_range_of_motion'],
    'realsense':[
        'test_REALSENSE_cable',
        'test_REALSENSE_frame_rate'
    ],
}



#!/usr/bin/env python3
import stretch_diagnostics.test_helpers as test_helpers
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.stepper
import stretch_body.hello_utils as hu
import os
import glob


class Test_SIMPLE_udev(unittest.TestCase):
    """
    Test USB Devices on Bus
    """
    test = TestBase('test_SIMPLE_udev')

    arduino_devices = ['hello-wacc',
                     'hello-motor-left-wheel',
                     'hello-pimu',
                     'hello-motor-arm',
                     'hello-motor-right-wheel',
                     'hello-motor-lift']

    dynamixel_devices= ['hello-dynamixel-head','hello-dynamixel-wrist']


    def test_udev_log(self):
        """
        Log udev files for Stretch devices.
        """
        udev_files_list= glob.glob('/etc/udev/rules.d/*hello*.rules')
        for p in udev_files_list:
            fn_no_path=p[p.rfind('/'):]
            new_fn=fn_no_path+'_%s.log'%self.test.timestamp
            os.system("cp %s %s"%(p,self.test.test_result_dir+new_fn))

        ls_l_file = self.test.test_result_dir+'/hello_usb_symlinks_%s.log'%self.test.timestamp
        os.system('ls -l /dev/hello* >>%s'%ls_l_file)

    def test_dynamixel_udev(self):
        """
        Check that all Dynamixels devices have valid UDEV mappings.
       """
        for s in self.dynamixel_devices:
            port = '/dev/' + s
            print('Checking UDEV for %s' % port)
            dp = hdu.is_device_present(port)
            self.assertTrue(dp, msg='Device %s not on bus' % s)
            udev_sn = test_helpers.get_serial_nos_from_udev('/etc/udev/rules.d/99-hello-dynamixel.rules', s)
            self.assertFalse(len(udev_sn) == 0,'UDEV rule for %s not found in /etc/udev/rules.d/99-hello-dynamixel.rules' % s)
            self.assertFalse(len(udev_sn) > 1,'Multiple UDEV rules for %s found in /etc/udev/rules.d/99-hello-dynamixel.rules' % s)

    def test_ftdi_device_count(self):
        """
        Check that there are two FTDI devices on bus.
        """
        tty_dev=test_helpers.find_tty_devices()
        dxl_keys=[]
        for k in tty_dev.keys():
            if tty_dev[k]['model']=='FT232R_USB_UART':
                dxl_keys.append(k)
        self.assertTrue(len(dxl_keys)==2,msg='Did not find two FTDI devices for Dynamixels')

    def test_ttyACM_symlinks(self):
        """
        Check that all ttyACM devices have symlinks.
        """
        # First check that there are at least 6 TTYACM devices
        ttyACM_SN_map = test_helpers.find_arduino_devices_sn()  # dict of {'/dev/ttyACM6': 'E469692150555733352E3120FF0A0C22',...}
        self.assertTrue(len(ttyACM_SN_map) >= 6,'Found less than 6 ttyACM devices')
        self.test.log_data('arduino_devices_udevadm', ttyACM_SN_map)

        # Now check that all ttyACM have symlinks
        for ttyACM in ttyACM_SN_map.keys():
            # Returns something like if mapped
            # ['/dev/hello-wacc',
            #  '/dev/serial/by-id/usb-Arduino_LLC_Hello_Wacc_C9D6FD8750555733352E3120FF0C0D1A-if00',
            #  '/dev/serial/by-path/pci-0000:00:14.0-usb-0:4.2.3.3:1.0']
            devlinks = test_helpers.extract_udevadm_info(ttyACM, 'DEVLINKS').split(' ')
            sn = test_helpers.extract_udevadm_info(ttyACM, 'ID_SERIAL_SHORT')
            print('Checking symlinks for %s' % ttyACM)
            # print('DEVLINKS',devlinks)
            if len(devlinks) < 3:
                self.test.log_data('missing_symlink_%s' % ttyACM, test_helpers.extract_udevadm_info(ttyACM))
            self.assertTrue(len(devlinks) == 3,'Device %s with serial %s lacks UDEV symlink.' % (ttyACM, sn))

    def test_arduino_udev(self):
        """
        Check that all arduino devices have valid UDEV mappings.
       """
        for s in self.arduino_devices:
            port='/dev/'+s
            print('Checking UDEV for %s'%port)
            dp = hdu.is_device_present(port)
            self.assertTrue(dp, msg='Device %s not on bus' % s)
            udev_sn = test_helpers.get_serial_nos_from_udev('/etc/udev/rules.d/95-hello-arduino.rules', s)
            self.assertFalse(len(udev_sn) == 0,'UDEV rule for %s not found in /etc/udev/rules.d/95-hello-arduino.rules' % s)
            self.assertFalse(len(udev_sn) > 1,'Multiple UDEV rules for %s found in /etc/udev/rules.d/95-hello-arduino.rules' % s)


test_suite = TestSuite(test=Test_SIMPLE_udev.test,failfast=False)
test_suite.addTest(Test_SIMPLE_udev('test_udev_log'))
test_suite.addTest(Test_SIMPLE_udev('test_ttyACM_symlinks'))
test_suite.addTest(Test_SIMPLE_udev('test_arduino_udev'))
test_suite.addTest(Test_SIMPLE_udev('test_ftdi_device_count'))
test_suite.addTest(Test_SIMPLE_udev('test_dynamixel_udev'))
if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import val_in_range
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import stretch_factory.hello_device_utils as hdu
from stretch_body.dynamixel_XL430 import DynamixelXL430, DynamixelCommError
import unittest
from stretch_body.robot_params import RobotParams
import glob


class Test_SIMPLE_accessories(unittest.TestCase):
    """
    Test to verify if the Stretch Accessories param configuration match the found HW devices
    """
    test = TestBase('test_SIMPLE_accessories')

    def test_check_tool_config(self):
        """
        Log Stretch robot tool configuration details from params
        """
        robot_tool = RobotParams.get_params()[1]['robot']['tool']
        robot_model = RobotParams.get_params()[1]['robot']['model_name']
        robot_batch = RobotParams.get_params()[1]['robot']['batch_name']
        self.test.log_data('robot_tool', robot_tool)
        self.test.log_data('robot_model', robot_model)
        self.test.log_data('robot_batch', robot_batch)

        print("Found robot tool config: {}".format(robot_tool))
        print("Found robot model config: {}".format(robot_model))
        print("Found robot batch config: {}".format(robot_batch))

    def test_verify_params(self):
        """
        Verify if the stretch robot tool configuration matches the recommended stretch params
        """
        ref_params = None
        standard_gripper_yaml_RE2V0 = {
            'params': [],
            'lift': {'i_feedforward': 1.2},
            'hello-motor-lift': {'gains': {'i_safety_feedforward': 1.2}}}
        standard_gripper_yaml_RE1V0 = {
            'params': [],
            'lift': {'i_feedforward': 0.54},
            'hello-motor-lift': {'gains': {'i_safety_feedforward': 0.4}}}
        dex_wrist_yaml_RE1V0 = {
            'params': ['stretch_tool_share.stretch_dex_wrist.params'],
            'lift': {'i_feedforward': 0.75},
            'hello-motor-lift': {'gains': {'i_safety_feedforward': 0.75}}}
        dex_wrist_yaml_RE2V0 = {
            'params': ['stretch_tool_share.stretch_dex_wrist.params'],
            'lift': {'i_feedforward': 1.8},
            'hello-motor-lift': {'gains': {'i_safety_feedforward': 1.8}}}

        fail_msg='\n'
        if self.test.data_dict['robot_tool'] == 'tool_stretch_gripper' and self.test.data_dict[
            'robot_model'] == 'RE1V0':
            ref_params = standard_gripper_yaml_RE1V0
        elif self.test.data_dict['robot_tool'] == 'tool_stretch_dex_wrist' and self.test.data_dict[
            'robot_model'] == 'RE1V0':
            ref_params = dex_wrist_yaml_RE1V0
            fail_msg=fail_msg+"Checkout stretch wrist tool guide https://docs.hello-robot.com/0.2/stretch-hardware-guides/docs/dex_wrist_guide_re1/"
        elif self.test.data_dict['robot_tool'] == 'tool_stretch_gripper' and self.test.data_dict[
            'robot_model'] == 'RE2V0':
            ref_params = standard_gripper_yaml_RE2V0
        elif self.test.data_dict['robot_tool'] == 'tool_stretch_dex_wrist' and self.test.data_dict[
            'robot_model'] == 'RE2V0':
            ref_params = dex_wrist_yaml_RE1V0
            fail_msg=fail_msg+"Checkout stretch wrist tool guide https://docs.hello-robot.com/0.2/stretch-hardware-guides/docs/dex_wrist_guide_re2/"
        self.assertIsNotNone(ref_params, "Unable to identify the right stretch tool configuration"+fail_msg)
        self.test.log_params('correct_params', ref_params)

        _params = []
        try:
            _params = RobotParams.get_params()[1]['params']
        except KeyError:
            _params = []
        self.assertEqual(ref_params['params'], _params, "'params' dont match")

        _lift_i_feedforward = RobotParams.get_params()[1]['lift']['i_feedforward']
        delta=abs(ref_params['lift']['i_feedforward']- _lift_i_feedforward)
        pct_diff = delta / ref_params['lift']['i_feedforward']
        msg="Lift i_feedforward of %f is far from nominal of %f. Differ by %f pct. \nYou may need to run REx_calibrate_gravity_comp.py."%\
            (ref_params['lift']['i_feedforward'],_lift_i_feedforward,pct_diff)
        self.assertTrue(pct_diff<0.2,msg=msg)

        _hello_motor_lift_i_safety_feedforward = RobotParams.get_params()[1]['hello-motor-lift']['gains'][
            'i_safety_feedforward']
        delta = abs(ref_params['hello-motor-lift']['gains']['i_safety_feedforward'] - _hello_motor_lift_i_safety_feedforward)
        pct_diff = delta / ref_params['hello-motor-lift']['gains']['i_safety_feedforward']
        msg = "Lift i_safety_feedforward of %f is far from nominal of %f. Differ by %f pct. \nYou may need to run REx_calibrate_gravity_comp.py." % \
              (ref_params['hello-motor-lift']['gains']['i_safety_feedforward'], _hello_motor_lift_i_safety_feedforward, pct_diff)
        self.assertTrue(pct_diff < 0.2, msg=msg)


    def get_servo_ids(self, port, baud_to=115200):
        found_ids = []
        print('\nScanning for servo at port: {}'.format(port))
        print('----------------------------------------------------------')
        b = baud_to
        for id in range(20):
            print("Checking at ID %d and baud %d" % (id, b))
            m = DynamixelXL430(id, port, baud=b)
            m.logger.disabled = True
            try:
                if m.startup():
                    print('Found servo at ID %d and Baud %d' % (id, b))
                    found_ids.append(id)
            except DynamixelCommError:
                print("ping failed for ID: " + str(id))
                continue
        return found_ids

    def test_check_wrist_dynamixels(self):
        """
         Check the number of dynamixels present in the wrist
        """
        dev_list = glob.glob('/dev/hello-dynamixel-*')
        print("Found Servo IDs: {}".format(dev_list))
        self.assertEqual(len(dev_list), 2, "Unable to find all hello-dynamixel-* devices")
        self.assertTrue('/dev/hello-dynamixel-wrist' in dev_list, "Unable to find device: /dev/hello-dynamixel-wrist")
        found_ids = self.get_servo_ids(port='/dev/hello-dynamixel-wrist')
        self.test.log_data('found_servo_ids', found_ids)

        if self.test.data_dict['robot_tool'] == 'tool_stretch_dex_wrist':
            self.assertEqual(len(found_ids), 4, "Unable to find the all the servos")
            found=13 in found_ids and 14 in found_ids and 15 in found_ids and  16 in found_ids
            msg = 'Invalid servo ids in wrist Dynamixel chain'
            self.assertTrue(found,msg=msg)

        if self.test.data_dict['robot_tool'] == 'tool_stretch_gripper':
            self.assertEqual(len(found_ids), 2, "Unable to find the all the servos")
            found = 13 in found_ids and 14 in found_ids
            msg = 'Invalid servo ids in wrist Dynamixel chain'
            self.assertTrue(found, msg=msg)


test_suite = TestSuite(test=Test_SIMPLE_accessories.test, failfast=False)
test_suite.addTest(Test_SIMPLE_accessories('test_check_tool_config'))
test_suite.addTest(Test_SIMPLE_accessories('test_verify_params'))
test_suite.addTest(Test_SIMPLE_accessories('test_check_wrist_dynamixels'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import val_in_range
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.dynamixel_hello_XL430
import stretch_body.dynamixel_XL430
class Test_SIMPLE_dynamixel_configure(unittest.TestCase):
    """
    Test Dynamixel device basic functionality
    """
    test = TestBase('test_SIMPLE_dynamixel_configure')
    devices=['hello-dynamixel-head','hello-dynamixel-wrist']
    joints=['head_tilt','head_pan','wrist_yaw','stretch_gripper']
    def test_device_present(self):
        """
        Check that Dynamixel devices are present
        """
        for d in self.devices:
            dh=hdu.is_device_present('/dev/'+d)
            self.assertTrue(dh,msg='Device %s not on bus. May be UDEV or hardware issue.'%d)

    def test_identify_baud(self):
        "Check that baud rates are set correctly"
        baud_id={'head_pan':{},'head_tilt':{},'wrist_yaw':{},'stretch_gripper':{}}
        baud_id['head_pan']['identified']=stretch_body.dynamixel_XL430.DynamixelXL430.identify_baud_rate(11,'/dev/hello-dynamixel-head')
        baud_id['head_tilt']['identified'] = stretch_body.dynamixel_XL430.DynamixelXL430.identify_baud_rate(12,'/dev/hello-dynamixel-head')
        baud_id['wrist_yaw']['identified'] = stretch_body.dynamixel_XL430.DynamixelXL430.identify_baud_rate(13,'/dev/hello-dynamixel-wrist')
        baud_id['stretch_gripper']['identified'] = stretch_body.dynamixel_XL430.DynamixelXL430.identify_baud_rate(14,'/dev/hello-dynamixel-wrist')

        for j in self.joints:
            servo=stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name=j, chain=None)
            baud_id[j]['params']=servo.params['baud']
            msg='Baud mismmatch for %s. YAML param is %d and servo is set to %d'%(j,baud_id[j]['params'],baud_id[j]['identified'])
            self.assertTrue(baud_id[j]['params'] == baud_id[j]['identified'],msg=msg)
        self.test.log_data('baud_identification', baud_id)

    def test_servo_ping(self):
        "Ping the primary Dynamixel servos"
        ping={}
        for j in self.joints:
            servo = stretch_body.dynamixel_hello_XL430.DynamixelHelloXL430(name=j, chain=None)
            st=servo.startup()
            self.assertTrue(st,'Unable to startup %s'%j)
            ping[j]=servo.do_ping()
            self.assertTrue(ping[j], msg='Failed to ping servo %s'%j)
        self.test.log_data('servo_pinged', ping)


test_suite = TestSuite(test=Test_SIMPLE_dynamixel_configure.test,failfast=False)
test_suite.addTest(Test_SIMPLE_dynamixel_configure('test_device_present'))
test_suite.addTest(Test_SIMPLE_dynamixel_configure('test_servo_ping'))
test_suite.addTest(Test_SIMPLE_dynamixel_configure('test_identify_baud'))
if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
import unittest
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_suite import TestSuite
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_helpers import Dmesg_monitor, get_rs_details
from colorama import Fore, Style
from subprocess import Popen, PIPE, STDOUT
import time
import numpy as np
import click
import stretch_body.robot
import subprocess
from tabulate import tabulate

# Enter each stream's pass fps rate
streams_assert = {'Depth': 29, 'Color': 29, 'Gyro': 199, 'Accel': 62}  # Refer create_config_target_*


def create_config_target_hi_res():
    f = open('/tmp/d435i_confg.cfg', "w+")
    config_script = ["DEPTH,1280,720,30,Z16,0",
                     "COLOR,1920,1080,30,RGB8,0",
                     "ACCEL,1,1,63,MOTION_XYZ32F",
                     "GYRO,1,1,200,MOTION_XYZ32F"]

    header = ["STREAM", "WIDTH", "HEIGHT", "FPS", "FORMAT", "STREAM_INDEX"]
    config_data = []
    for ll in config_script:
        f.write(ll + "\n")
        config_data.append(ll.split(','))
    config_table = str(tabulate(config_data, headers=header, tablefmt='github')).split('\n')

    f.close()
    target = {'duration': 31,
              'nframe': 900,
              'margin': 16,
              'streams': {
                  'Color': {'target': 900, 'sampled': 0},
                  'Depth': {'target': 900, 'sampled': 0},
                  'Accel': {'target': 900, 'sampled': 0},
                  'Gyro': {'target': 900, 'sampled': 0}}}
    return target


def create_config_target_low_res():
    f = open('/tmp/d435i_confg.cfg', "w+")
    config_script = ["DEPTH,424,240,30,Z16,0",
                     "COLOR,424,240,30,RGB8,0",
                     "ACCEL,1,1,63,MOTION_XYZ32F",
                     "GYRO,1,1,200,MOTION_XYZ32F"]

    header = ["STREAM", "WIDTH", "HEIGHT", "FPS", "FORMAT", "STREAM_INDEX"]
    config_data = []
    for ll in config_script:
        f.write(ll + "\n")
        config_data.append(ll.split(','))
    config_table = str(tabulate(config_data, headers=header, tablefmt='github')).split('\n')

    f.close()
    target = {'duration': 31,
              'nframe': 900,
              'margin': 16,
              'streams': {
                  'Color': {'target': 900, 'sampled': 0},
                  'Depth': {'target': 900, 'sampled': 0},
                  'Accel': {'target': 900, 'sampled': 0},
                  'Gyro': {'target': 900, 'sampled': 0}}}
    return target


known_msgs = [[0, 30, 'uvcvideo: Failed to query (GET_CUR) UVC control'],
              [0, 15, 'Non-zero status (-71) in video completion handler'],
              [0, 15, 'No report with id 0xffffffff found'],
              [0, 10, 'uvcvideo: Found UVC 1.50 device Intel(R) RealSense(TM) Depth Camera 435'],
              [0, 5, 'uvcvideo: Unable to create debugfs'],
              [0, 4, 'hid-sensor-hub'],
              [0, 6, 'input: Intel(R) RealSense(TM) Depth Ca'],
              [0, 1, 'uvcvideo: Failed to resubmit video URB (-1).'],
              [0, 1, 'Netfilter messages via NETLINK v0.30.'],
              [0, 1, 'USB disconnect']]


def get_frame_id_from_log_line(stream_type, line):
    if line.find(stream_type) != 0:
        return None
    return int(line.split(',')[2])


def get_rs_data(target):
    cmd = 'rs-data-collect -c /tmp/d435i_confg.cfg -f /tmp/d435i_log.csv -t %d -m %d' % (
        target['duration'], target['nframe'])
    out = Popen(cmd, shell=True, bufsize=64, stdin=PIPE, stdout=PIPE, close_fds=True).stdout.read().decode("utf-8")
    ff = open('/tmp/d435i_log.csv')
    data = ff.readlines()
    data = data[10:]  # drop preamble
    return data


def get_fps(data, stream, t):
    timestamps = []
    for ll in data:
        tag = stream + ',' + t
        if tag in ll:
            l = ll.split(',')[-1].split('\n')[0]
            if stream == 'Accel' or stream == 'Gyro':
                l = ll.split(',')[-4].split('\n')[0]
            timestamp = float(l) / 1000
            timestamps.append(timestamp)
    if len(timestamps) > 1:
        duration = timestamps[-1] - timestamps[0]
        avg_fps = len(timestamps) / duration
        return avg_fps
    else:
        return 000.0


def check_FPS(data):
    fps_dict = {}
    for s in streams_assert.keys():
        fps = get_fps(data, s, '0')
        fps_dict[s] = fps
        if fps > streams_assert[s]:
            print(Fore.GREEN + '[Pass] %s Rate : %f FPS' % (s, fps) + Style.RESET_ALL)
        else:
            print(Fore.RED + '[Fail] %s Rate : %f FPS < %d FPS' % (s, fps, streams_assert[s]) + Style.RESET_ALL)
    return fps_dict


def check_frames_collected(data, target):
    frames_n_dict = {}
    for ll in data:
        for kk in target['streams'].keys():
            id = get_frame_id_from_log_line(kk, ll)
            if id is not None:
                target['streams'][kk]['sampled'] = max(id, target['streams'][kk]['sampled'])
    for kk in target['streams'].keys():
        sampled_frames = target['streams'][kk]['sampled']
        min_frames = target['streams'][kk]['target'] - target['margin']
        frames_n_dict[kk] = {'sampled_frames': sampled_frames, 'min_frames': min_frames}
        if sampled_frames >= min_frames:
            print(Fore.GREEN + '[Pass] Stream: %s with %d frames collected' % (kk, sampled_frames))
        else:
            print(Fore.RED + '[Fail] Stream: %s with %d frames of %d collected' % (kk, sampled_frames, min_frames))
    print(Style.RESET_ALL)
    return frames_n_dict


class Test_REALSENSE_frame_rate(unittest.TestCase):
    """
    Testing the frame rate capture performance of realsense
    """

    # test object is always expected within a TestCase Class
    test = TestBase('test_REALSENSE_frame_rate')
    test.add_hint("Realsense frame rate issues can sometimes be solved by relasense drivers reinstall or firmware "
                  "update.")

    @classmethod
    def setUpClass(self):
        dmesg_log_fn = "{}/{}_{}.log".format(self.test.results_directory_test_specific,
                                             "dmesg",
                                             self.test.timestamp)
        self.dmesg = Dmesg_monitor(print_new_msg=True, log_fn=dmesg_log_fn)
        self.dmesg.start()

    @classmethod
    def tearDownClass(self):
        self.dmesg.stop()
        print("\nCollected DMESG")
        print("---------------")
        for l in self.dmesg.output_list:
            print(l)

    def test_USB3_2_connection(self):
        """
        Check that Realsense camera is on USB3.2 connection
        """
        out = Popen("rs-enumerate-devices| grep Usb | grep 3.2", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                    close_fds=True).stdout.read()

        if len(out):
            print('Confirmed USB 3.2 connection to realsense device')
        else:
            self.add_hint('Did not find USB 3.2 connection to realsense device')
        self.assertIsNot(len(out), 0, msg='Did not find USB 3.2 connection to realsense device')

    def test_realsense_on_usb_bus(self):
        """
        Check that Realsense camera is on USB bus.
        """
        print('---- Checking for Intel D435i ----')
        cmd = "lsusb -d 8086:0b3a"
        returned_value = subprocess.call(cmd, shell=True)  # returns the exit code in unix
        if returned_value != 0:
            self.test.add_hint('Realsense D435i not found at USB Bus')
        self.assertEqual(returned_value, 0)

    def test_realsense_details(self):
        """
        Capture realsense details and log
        """
        d = get_rs_details()
        if d is None:
            self.test.add_hint('Not able to launch Realsense driver. It may be conflicting with ROS')
        self.assertIsNotNone(d)
        self.test.log_data('realsense_details', d)

    def test_get_realsense_drivers_info(self):
        """
        Collect realsense driver information
        """
        driver_info = {}

        fw_details = Popen("rs-fw-update -l | grep -i 'firmware'", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                           close_fds=True).stdout.read().decode()
        fw_details = fw_details.split(',')[3]
        fw_version = fw_details.split(' ')[-1]
        driver_info['fw_version'] = fw_version

        nuc_bios_version = Popen("sudo dmidecode -s bios-version", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                                 close_fds=True).stdout.read().decode().rstrip()
        system_version = Popen("sudo dmidecode -s system-version", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                               close_fds=True).stdout.read().decode().rstrip()
        baseboard_version = Popen("sudo dmidecode -s baseboard-version", shell=True, bufsize=64, stdin=PIPE,
                                  stdout=PIPE, close_fds=True).stdout.read().decode().rstrip()
        processor_version = Popen("sudo dmidecode -s processor-version", shell=True, bufsize=64, stdin=PIPE,
                                  stdout=PIPE, close_fds=True).stdout.read().decode().rstrip()
        kernel_version = Popen("uname -r", shell=True, bufsize=64, stdin=PIPE, stdout=PIPE,
                               close_fds=True).stdout.read().decode().rstrip()

        driver_info['nuc_bios_version'] = nuc_bios_version
        driver_info['system_version'] = system_version
        driver_info['baseboard_version'] = baseboard_version
        driver_info['processor_version'] = processor_version
        driver_info['firmware_vekernel_versionrsion'] = kernel_version

        # check_install_v4l2()
        print('\nD435i Firmware version: %s\n' % (fw_version))
        print("Linux Kernel Version : %s" % (kernel_version))
        print("NUC Bios Version : %s" % (nuc_bios_version))
        print("NUC System Version : %s" % (system_version))
        print("NUC Baseboard Version : %s" % (baseboard_version))
        print("Processor Version : %s" % (processor_version))
        self.test.log_data('system_info', driver_info)

    def test_frame_rate_high_res(self):
        """
        Check the frames collected and frame rate achieved while collecting high res rs data
        """
        target = create_config_target_hi_res()
        self.test.log_params("high_res_config", target)

        print("Collecting High res data from realsense......")
        data = get_rs_data(target)
        frames_collected = check_frames_collected(data, target)
        fps_dict = check_FPS(data)
        self.test.log_data("high_res_frames_collected", frames_collected)
        self.test.log_data("high_res_frame_rates", fps_dict)

        for kk in frames_collected.keys():
            self.assertGreaterEqual(frames_collected[kk]['sampled_frames'], frames_collected[kk]['min_frames'])
        for s in streams_assert.keys():
            self.assertGreater(fps_dict[s], streams_assert[s])

    def test_frame_rate_low_res(self):
        """
        Check the frames collected and frame rate achieved while collecting low res rs data
        """
        target = create_config_target_low_res()
        self.test.log_params("high_res_config", target)

        print("Collecting Low res data from realsense......")
        data = get_rs_data(target)
        frames_collected = check_frames_collected(data, target)
        fps_dict = check_FPS(data)
        self.test.log_data("low_res_frames_collected", frames_collected)
        self.test.log_data("low_res_frame_rates", fps_dict)

        for kk in frames_collected.keys():
            self.assertGreaterEqual(frames_collected[kk]['sampled_frames'], frames_collected[kk]['min_frames'])
        for s in streams_assert.keys():
            self.assertGreater(fps_dict[s], streams_assert[s])

    def check_dmesg(self, msgs):
        unknown_msgs = []
        excessive_msgs = []
        unexpected_msgs = []
        no_error = True
        for m in msgs:
            if len(m):
                found = False
                for i in range(len(known_msgs)):
                    if m.find(known_msgs[i][2]) != -1:
                        found = True
                        known_msgs[i][0] = known_msgs[i][0] + 1
                if not found:
                    unknown_msgs.append(m)
        for i in range(len(known_msgs)):
            if known_msgs[i][0] >= known_msgs[i][1]:
                print(Fore.YELLOW + '[Warning] Excessive dmesg warnings (%d) of: %s' % (
                    known_msgs[i][0], known_msgs[i][2]))
                excessive_msgs.append(known_msgs[i][2])
                no_error = False
        if len(unknown_msgs):
            print('[Warning] Unexpected dmesg warnings (%d)' % len(unknown_msgs))
            unexpected_msgs = unknown_msgs
            no_error = False
            for i in unknown_msgs:
                print(i)
        if no_error:
            print(Fore.GREEN + '[Pass] No unexpected dmesg warnings')
        print(Style.RESET_ALL)
        self.test.log_data("excessive_dmesgs", excessive_msgs)
        self.test.log_data("unexpected_dmesgs", unexpected_msgs)
        self.assertTrue(no_error, "Errors captured in DMESGs.")

    def test_check_dmesgs(self):
        """
        Check Dmesg output errors
        """
        self.check_dmesg(self.dmesg.get_output_list())


test_suite = TestSuite(test=Test_REALSENSE_frame_rate.test, failfast=False)
test_suite.addTest(Test_REALSENSE_frame_rate('test_USB3_2_connection'))
test_suite.addTest(Test_REALSENSE_frame_rate('test_realsense_on_usb_bus'))
test_suite.addTest(Test_REALSENSE_frame_rate('test_get_realsense_drivers_info'))
test_suite.addTest(Test_REALSENSE_frame_rate('test_realsense_details'))
test_suite.addTest(Test_REALSENSE_frame_rate('test_frame_rate_high_res'))
test_suite.addTest(Test_REALSENSE_frame_rate('test_frame_rate_low_res'))
test_suite.addTest(Test_REALSENSE_frame_rate('test_check_dmesgs'))

if __name__ == '__main__':
    runner = TestRunner(suite=test_suite, doc_verify_fail=False)
    runner.run()



#!/usr/bin/env python3

from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import requests
import socket
class Test_SIMPLE_wifi(unittest.TestCase):
    """
    Test if Wifi is working reliably
    """
    test = TestBase('test_SIMPLE_wifi')


    def test_wifi_on(self):
        """
        Test that WiFi is on and internet connection works
        """
        try:
            s = socket.create_connection(("www.hello-robot.com", 80))
            if s is not None:
                s.close()
        except OSError:
            self.assertTrue(False,msg='Internet connection is bad. Check Wifi settings')



test_suite = TestSuite(test=Test_SIMPLE_wifi.test,failfast=False)
test_suite.addTest(Test_SIMPLE_wifi('test_wifi_on'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
import unittest
import os
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_suite import TestSuite
from stretch_diagnostics.test_runner import TestRunner


class Test_ROS_calibration(unittest.TestCase):
    """
    Test to check if calibrated urdf and head calibration file exist in workspace.
    """

    test = TestBase('test_ROS_calibration')

    @classmethod
    def setUpClass(self):
        # Write Test Class optional startup Scripts
        # E.g:
        # r = stretch_body.robot.Robot()
        # r.startup()
        pass

    @classmethod
    def tearDownClass(self):
        # Write Test Class optional ending Scripts
        # E.g:
        # r.stop()
        pass

    def test_head_calib_file(self):
        """
        Check if head calibration file exists in workspace
        """
        
        ros_distro = os.getenv('ROS_DISTRO')
        home_path = os.getenv('HOME')
        yaml_catkin_ws = 'catkin_ws/src/stretch_ros/stretch_core/config/controller_calibration_head.yaml'
        yaml_ament_ws = 'ament_ws/src/stretch_ros2/stretch_description/urdf/controller_calibration_head.yaml'

        head_calibration_path='./'
        if ros_distro in ['melodic', 'noetic']:
            head_calibration_path = '{0}/{1}'.format(home_path, yaml_catkin_ws)
        elif ros_distro in ['galactic', 'humble']:
            head_calibration_path = '{0}/{1}'.format(home_path, yaml_ament_ws)
        else:
            self.assertTrue(0,"No ROS distro sourced. Source ROS in the ~/.bashrc file.")
        
        head_calibration_exists = os.path.exists(head_calibration_path)

        if not head_calibration_exists:
            self.assertTrue(0,"Could not find head calibration file {}.".format(head_calibration_path))
        
        self.test.log_data('head_calibration_exists', head_calibration_exists)
        self.assertTrue(head_calibration_exists,'Head calibration file does not exist')

    def test_calib_urdf_file(self):
        """
        Check if calibrated urdf file exists in workspace
        """

        ros_distro = os.getenv('ROS_DISTRO')
        home_path = os.getenv('HOME')
        urdf_catkin_ws = 'catkin_ws/src/stretch_ros/stretch_description/urdf/stretch.urdf'
        urdf_ament_ws = 'ament_ws/src/stretch_ros2/stretch_description/urdf/stretch.urdf'

        calibrated_urdf_path='./'
        if ros_distro in ['melodic', 'noetic']:
            calibrated_urdf_path = '{0}/{1}'.format(home_path, urdf_catkin_ws)
        elif ros_distro in ['galactic', 'humble']:
            calibrated_urdf_path = '{0}/{1}'.format(home_path, urdf_ament_ws)
        else:
            self.assertTrue(0,"No ROS distro sourced. Source ROS in the ~/.bashrc file.")
        
        calibrated_urdf_exists = os.path.exists(calibrated_urdf_path)

        if not calibrated_urdf_exists:
            self.assertTrue(0,"Could not find calibrated URDF file {}.".format(calibrated_urdf_path))
        
        self.test.log_data('calibrated_urdf_exists', calibrated_urdf_exists)
        self.assertTrue(calibrated_urdf_exists, 'Calibrated URDF does not exist')

test_suite = TestSuite(test=Test_ROS_calibration.test, failfast=False)

test_suite.addTest(Test_ROS_calibration('test_head_calib_file'))
test_suite.addTest(Test_ROS_calibration('test_calib_urdf_file'))

if __name__ == '__main__':
    runner = TestRunner(suite=test_suite, doc_verify_fail=False)
    runner.run()



#!/usr/bin/env python3

import os
import sys

import yaml
from stretch_diagnostics.test_order import test_order
from colorama import Fore, Style
import importlib
from stretch_diagnostics.test_helpers import confirm
import stretch_body.hello_utils as hu
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_helpers import command_list_exec, get_installed_package_info, center_string
import click
import glob
from zipfile import ZipFile


class TestManager():
    def __init__(self, test_type):
        self.next_test_ready = False
        self.test_type = test_type

        self.pkg_tests_path = '{}/stretch_diagnostics_tests/{}_tests'.format(
            get_installed_package_info('hello-robot-stretch-diagnostics')['path'], test_type)
        sys.path.append(self.pkg_tests_path)

        # Get Fleet ID
        self.fleet_id = os.environ['HELLO_FLEET_ID']

        results_directory = os.environ['HELLO_FLEET_PATH'] + '/log/diagnostic_check'
        self.test_timestamp = hu.create_time_string()
        self.tests_order = test_order[test_type]
        self.DiagnosticCheck_filename = 'diagnostic_check_%s_%s_%s.yaml' % (
            self.test_type, self.fleet_id, self.test_timestamp)
        self.results_directory = results_directory
        self.system_health_dict = {'total_tests': 0,
                                   'total_tests_failed': 0,
                                   'all_success': False,
                                   'check_timestamp': self.test_timestamp,
                                   'robot_name': self.fleet_id,
                                   'tests': None
                                   }
        self.disable_print_warning = False
        self.disable_print_error = False

    def get_TestModule(self, test_name):
        try:
            TestCase = importlib.import_module(test_name)
            return TestCase
        except ModuleNotFoundError:
            self.print_error('Unable to Load Test : {}'.format(test_name))
            return None

    def get_TestSuite(self, test_name):
        try:
            test_module = importlib.import_module(test_name)
            return test_module.test_suite
        except Exception as e:
            self.print_error('Unable to Load Test Suite: {}\n ERROR: {}'.format(test_name, e))
            return None

    def run_test(self, test_name):
        try:
            test_suite = self.get_TestSuite(test_name)
            runner = TestRunner(test_suite, False)
            runner.run()
            result = self.read_latest_test_result(test_name)
            del sys.modules[test_name]
            return result
        except Exception as e:
            self.print_error('Unable to Run Test: {} \n Error: {}'.format(test_name, e))
            return None

    def get_latest_files_with_extension(self, test_name, ext):
        # Retrieve the latest files of type test_name with extension ext
        # Not all tests will also generate a file of type ext
        try:
            list_of_files = []
            all_files = glob.glob(self.results_directory + '/' + test_name + '/' + '*.' + ext)
            unique_files = []
            for a in all_files:
                base = a[:a.rfind('_')]
                if not base in unique_files:
                    unique_files.append(base)
            for u in unique_files:
                all_files = glob.glob(u + '*.' + ext)
                all_files.sort()
                list_of_files.append(all_files[-1])
            return list_of_files
        except:
            return []

    def get_latest_test_result_filename(self, test_name):
        try:
            listOfFiles = glob.glob(self.results_directory + '/' + test_name + '/' + test_name + '*.yaml')
            listOfFiles.sort()
            return listOfFiles[-1]
        except Exception as e:
            # self.print_warning('{} result file not found'.format(test_name))
            return None

    def read_latest_test_result(self, test_name):
        try:
            listOfFiles = glob.glob(self.results_directory + '/' + test_name + '/' + test_name + '*.yaml')
            listOfFiles.sort()
            with open(listOfFiles[-1]) as f:
                result = yaml.safe_load(f)
            return result
        except Exception as e:
            self.print_warning('{} result not found or not performed'.format(test_name))
            return None

    def run_all_test(self):
        end = False
        i = 0
        while not end:
            choice, i = self.print_run_all_choices(i)
            if confirm('Run the Test {}'.format(Fore.CYAN + choice + Style.RESET_ALL)):
                print('\n')
                self.run_test(choice)
                print('\n\n')
            i = i + 1
            if i == len(self.tests_order) or i > len(self.tests_order):
                end = True
            print('---------------------------------------------------------\n')
        self.generate_last_system_health_report()

    def archive_all_tests_status(self):
        for test_name in self.tests_order:
            self.archive_test_status(test_name)

    def archive_test_status(self, test_name):
        wd = self.results_directory + '/' + test_name
        if not os.path.isdir(wd):
            return
        if not os.path.isdir(wd + '/archive'):
            os.system('mkdir {}/archive'.format(wd))
        command_list_exec(["cd {}".format(wd),
                           "find './' -maxdepth 1 -not -type d -exec mv -t 'archive/' -- '{}' +"])

    def run_suite(self):
        for t in self.tests_order:
            self.run_test(t)

    def run_menu(self, show_subtests=False):
        while True:
            self.print_status_report(show_subtests)
            print(Style.BRIGHT + '############### MENU ################' + Style.RESET_ALL)
            print('Enter test # to run (q to quit)')
            print('A: archive tests')
            print('-------------------------------------')
            try:
                r = input()
                if r == 'q' or r == 'Q':
                    return
                elif r == 'A':
                    self.archive_all_tests_status()
                    print('Resetting All Tests')
                    print('-------------------------------------')
                else:
                    n = int(r)
                    if 0 <= n < len(self.tests_order):
                        test_name = self.tests_order[n]
                        print('Running test: %s' % test_name)
                        print('')
                        self.run_test(test_name)
                        print('########### TEST COMPLETE ##########\n')
                    else:
                        print('Invalid entry')
            except(TypeError, ValueError):
                print('Invalid entry')

    def print_run_all_choices(self, test_i):
        print('---------------------------------------------------------\n')
        print(Style.BRIGHT + 'Choose an action from the list:')
        print(
            '[0]  RUN TEST = [{}. {}]'.format(test_i + 1, Fore.CYAN + self.tests_order[test_i] + Fore.RESET))
        if not test_i == 0:
            print('[1]  RE-RUN PREVIOUS TEST = [{}. {}]'.format(test_i, self.tests_order[test_i - 1]))
        print('[2]  ABORT HEALTH TEST' + Style.RESET_ALL)
        choice = input('Enter the Choice:\n')
        if choice == '0':
            return self.tests_order[test_i], test_i
        elif choice == '1':
            if not test_i == 0:
                return self.tests_order[test_i - 1], test_i - 1
            else:
                return self.print_run_all_choices(test_i)
        elif choice == '2':
            print('ABORTING HEALTH TEST.....')
            sys.exit()
        else:
            return self.print_run_all_choices(test_i)

    def print_subtests_status(self,result):
        for stest in result['test_status']['subtests_status'].keys():
            description = result['test_status']['subtests_status'][stest]['description']
            status = result['test_status']['subtests_status'][stest]['status']
            if status == 'PASS':
                description = self.set_subtests_descripton(result, stest, status)
                click.secho(f"\t\t[{status}] {stest}: {description}", fg="green")
            else:
                description = self.set_subtests_descripton(result,stest,status)
                click.secho(f"\t\t[{status}] {stest}: {description}", fg="red")

    def set_subtests_descripton(self, result, stest, status):
        description = result['test_status']['subtests_status'][stest]['description']

        if status == 'FAIL':
            def parse_traceback(tb):
                assertion_line = tb[-2]
                tbl = assertion_line.split(':')
                if len(tbl) > 3:
                    return f"{tbl[-2]}:{tbl[-1]}"
                return tbl[-1]

            for item in result['FAILS']:
                k = list(item.keys())[0]
                if k == stest:
                    description = description + f"\n\t\t\tfailure: {parse_traceback(item[k])}"
            for item in result['ERRORS']:
                k = list(item.keys())[0]
                if k == stest:
                    description = description + f"\n\t\t\terror: {parse_traceback(item[k])}"
        elif status == 'PASS':
            pass
        return description

    def print_status_report(self,show_subtests=False):
        self.disable_print_warning = True
        self.disable_print_error = True
        i = 0
        for test_name in self.tests_order:
            result = self.read_latest_test_result(test_name)
            if not result:
                print(click.style('[%d] %s: Test result: N/A' % (i, test_name), fg="yellow", bold=True))
            elif result['test_status']['status'] != 'SUCCESS':
                print(click.style('[%d] %s: Test result: FAIL' % (i, test_name,), fg="red", bold=True))
                if show_subtests:
                    self.print_subtests_status(result)
            else:
                print(click.style('[%d] %s: Test result: PASS' % (i, test_name), fg="green", bold=True))
                if show_subtests:
                    self.print_subtests_status(result)
            i = i + 1
        self.disable_print_warning = False
        self.disable_print_error = False

    def generate_latest_zip_file(self, zip_file=None):

        if zip_file is None:
            zip_file = self.results_directory + '/diagnostic_check_%s_%s.zip' % (self.fleet_id, self.test_timestamp)

        # Pass in a zip file to append new tests

        files_to_zip = [self.results_directory + '/' + self.DiagnosticCheck_filename]
        rel_path_directory = 'diagnostic_'
        files_rel_path = [self.DiagnosticCheck_filename]

        for test_name in self.tests_order:
            fn = self.get_latest_test_result_filename(test_name)
            lfn = self.get_latest_files_with_extension(test_name, 'log')
            ifn = self.get_latest_files_with_extension(test_name, 'png')
            if (fn):
                files_to_zip.append(fn)
                files_rel_path.append(fn.split('/')[-2] + '/' + fn.split('/')[-1])
            for l in lfn:
                files_to_zip.append(l)
                files_rel_path.append(l.split('/')[-2] + '/' + l.split('/')[-1])
            for i in ifn:
                files_to_zip.append(i)
                files_rel_path.append(i.split('/')[-2] + '/' + i.split('/')[-1])
        try:
            with ZipFile(zip_file, 'a') as z:
                for file, fn_path in zip(files_to_zip, files_rel_path):
                    print('Adding: %s' % file)
                    z.write(file, fn_path)
        except FileNotFoundError:
            self.print_error("Unable zip the file {}".format(zip_file))
        return zip_file

    def generate_last_diagnostic_report(self, silent=False):
        tests_results_collection = []
        total_fail = 0
        total_tests = len(self.tests_order)
        total_tests_ran = 0
        all_success = True
        for test_name in self.tests_order:
            result = self.read_latest_test_result(test_name)
            if (result):
                result_status = result['test_status']
                tests_results_collection.append({test_name: result_status})
                if result_status['status'] != 'SUCCESS':
                    total_fail = total_fail + 1
                total_tests_ran = total_tests_ran + 1
        self.system_health_dict['tests'] = tests_results_collection
        self.system_health_dict['total_tests'] = total_tests
        self.system_health_dict['total_tests_ran'] = total_tests_ran
        self.system_health_dict['total_tests_failed'] = total_fail
        if total_tests_ran == 0:
            if not silent:
                self.print_error('Zero Tests were Ran.')
                print(Fore.RED)
            self.system_health_dict['all_success'] = False
        elif total_fail == 0:
            if not silent:
                print(Fore.GREEN)
            self.system_health_dict['all_success'] = True
        else:
            if not silent:
                print(Fore.YELLOW)
        try:
            with open(self.results_directory + '/' + self.DiagnosticCheck_filename, 'w') as file:
                documents = yaml.dump(self.system_health_dict, file)
        except FileNotFoundError:
            self.print_error(
                "Unable to find file {}".format(self.results_directory + '/' + self.DiagnosticCheck_filename))
        if not silent:
            print('\n\n')
            print('Last Diagnostic Report:')
            print('================================')
            print(yaml.dump(self.system_health_dict))
            print('\nReported {} Fails.'.format(total_fail))
            print(Style.RESET_ALL)
            print('Diagnostic Check Saved to : {}'.format(self.results_directory + '/' + self.DiagnosticCheck_filename))

        return self.system_health_dict

    def list_ordered_tests(self, verbosity=1):
        all_tests_dict = {}
        txt = "Printing Orderded TestSuites  for %s and it's included Sub-TestCases" % self.test_type.upper()
        print(Style.BRIGHT + txt)
        print('-' * len(txt) + '\n' + Style.RESET_ALL)
        for i in range(len(self.tests_order)):
            test_name = self.tests_order[i]
            all_tests_dict[test_name] = {}
            test_suite = self.get_TestSuite(test_name)
            if test_suite:
                sub_tests = test_suite._tests
                if not len(sub_tests):
                    self.print_warning('Unable Discover tests added to {}.test_suite'.format(test_name))
                else:
                    cls_doc = sub_tests[0].__doc__
                    if cls_doc:
                        cls_doc = cls_doc.strip()
                        if verbosity == 2:
                            print('  ' + cls_doc)
                        all_tests_dict[test_name]['Description'] = cls_doc
                    else:
                        if verbosity == 2:
                            self.print_warning('Test Class level Doc missing.')
                    for j in range(len(sub_tests)):
                        sub_test_name = sub_tests[j].id().split('.')[-1]
                        all_tests_dict[test_name][sub_test_name] = {'Description': None}
                        if verbosity == 1 or verbosity == 2:
                            print('  {} {}'.format('-', sub_test_name))
                        sub_test_description = sub_tests[j].shortDescription()
                        if sub_test_description:
                            all_tests_dict[test_name][sub_test_name] = {'Description': sub_test_description}
                            if verbosity == 2:
                                print('       {}'.format(sub_test_description))
                        else:
                            if verbosity == 2:
                                self.print_warning('Short Description not provided.', 7)

        all_test_list_ordereded = []
        for test_name in self.tests_order:
            all_test_list_ordereded.append({test_name: all_tests_dict[test_name]})

        return all_test_list_ordereded

    def print_warning(self, text, indent=0):
        if not self.disable_print_warning:
            e = '[WARNING]:'
            print(' ' * indent + Fore.YELLOW + e + text + Style.RESET_ALL)

    def print_error(self, text, indent=0):
        if not self.disable_print_error:
            e = '[ERROR]:'
            print(' ' * indent + Fore.RED + e + text + Style.RESET_ALL)



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import val_in_range
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.wacc
import stretch_body.hello_utils as hu

class Test_SIMPLE_wacc(unittest.TestCase):
    """
    Test USB Devices on Bus
    """
    test = TestBase('test_SIMPLE_wacc')

    def test_wacc_present(self):
        """
        Check that device is present
        """
        dp=hdu.is_device_present('/dev/hello-wacc')
        self.assertTrue(dp,msg='Device /dev/hello-wacc not present')
        ttyACMx=hdu.get_all_ttyACMx()
        self.test.log_data('ttyACMx_devices', ttyACMx)
        d=stretch_body.wacc.Wacc()
        su=d.startup()
        self.assertTrue(su,msg='Device /dev/hello-wacc found but not able to startup')
        d.stop()

    def test_wacc_sensors(self):
        """
        Check wacc sensors have reasonable values
        """
        d = stretch_body.wacc.Wacc()
        self.assertTrue(d.startup(),'Wacc startup failed.')
        d.pull_status()
        self.assertFalse(d.status['ax']==0,'AX of 0. Possible communication failure with accelerometer.')
        self.assertTrue(val_in_range('AX', d.status['ax'], vmin=9.0, vmax=10.5),msg='AX value of %f outside of bounds'%d.status['ax'])
        d.stop()

test_suite = TestSuite(test=Test_SIMPLE_wacc.test,failfast=True)
test_suite.addTest(Test_SIMPLE_wacc('test_wacc_present'))
test_suite.addTest(Test_SIMPLE_wacc('test_wacc_sensors'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
import unittest
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_suite import TestSuite
from stretch_diagnostics.test_runner import TestRunner

"""
This is a Template script for writing a SIMPLE type Test and generating a Test suite.
"""


class Test_SIMPLE_foo(unittest.TestCase):
    """
    Class level doc string describing this SIMPLE type TestCase class
    """

    # test object is always expected within a TestCase Class
    test = TestBase('test_SIMPLE_foo')

    @classmethod
    def setUpClass(self):
        # Write Test Class optional startup Scripts
        # E.g:
        # r = stretch_body.robot.Robot()
        # r.startup()
        pass

    @classmethod
    def tearDownClass(self):
        # Write Test Class optional ending Scripts
        # E.g:
        # r.stop()
        pass

    def test_foo_1(self):
        """
        Short Description of Test foo 1

        """
        # a = 10 * (1 / 0)

        self.test.log_params('param_key1', 'any value')
        self.test.log_data('data_key1', 100)
        self.assertEqual(1, 0,msg="Foo 1 msg.")

    def test_foo_2(self):
        """
        Short Description of Test foo 2
        """
        self.test.log_params('param_key2', {'val': 0})
        self.assertTrue(False,msg='Foo 2 msg')

    def test_foo_3(self):
        """
        Short Description of Test foo 3
        """
        self.assertTrue(96 > len('hello'),msg='Foo 3 msg')
        print(self.test._non_existing_object)

    def test_foo_4(self):
        """
        Short Description of Test foo 4
        """
        self.assertTrue(3 > len('hello'), msg="I have an error message")

    def test_foo_5(self):
        """
        Short Description of Test foo 5
        """
        self.assertGreaterEqual(10, 10.5, msg="I have an error message")

    def test_foo_6(self):
        """
        Short Description of Test foo 6
        """
        self.assertGreaterEqual(10, 10, msg="I have an error message")


# Initialize you test suite with TestSuite(test,failfast)
# Arguments:
#   test = Test Object inside TestCase Class
#   failfast = If "True" the test exits at the first failiure. Use it for tests-order sensitive tests.
test_suite = TestSuite(test=Test_SIMPLE_foo.test, failfast=False)

# Add tests from the Test Class to the test_suite in the same order it would be run.
test_suite.addTest(Test_SIMPLE_foo('test_foo_1'))
test_suite.addTest(Test_SIMPLE_foo('test_foo_2'))
test_suite.addTest(Test_SIMPLE_foo('test_foo_3'))
test_suite.addTest(Test_SIMPLE_foo('test_foo_4'))
test_suite.addTest(Test_SIMPLE_foo('test_foo_5'))
test_suite.addTest(Test_SIMPLE_foo('test_foo_6'))

if __name__ == '__main__':
    # Create a TestRunner object with TestRunner(suite,doc_verify_fail)
    # Arguments:
    #   suite = test_suite built
    #   doc_verify_fail = If "True" , checks if Class level and Test level docstring descriptions available
    #                     If not available reports the missing description and exits the script.
    runner = TestRunner(suite=test_suite, doc_verify_fail=False)
    runner.run()



#!/usr/bin/env python3

import unittest
import yaml
import os, fnmatch
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import serial
import fcntl

class Test_SIMPLE_stretch_body_background(unittest.TestCase):
    """
    Test USB Devices on Bus
    """
    test = TestBase('test_SIMPLE_stretch_body_background')

    def test_stretch_body_background(self):
        """
        Check for background processes blocking serial ports
        """
        robot_devices = {'/dev/hello-wacc': False,
                         '/dev/hello-motor-left-wheel': False,
                         '/dev/hello-pimu': False,
                         '/dev/hello-lrf': False,
                         '/dev/hello-dynamixel-head': False,
                         '/dev/hello-dynamixel-wrist': False,
                         '/dev/hello-motor-arm': False,
                         '/dev/hello-motor-right-wheel': False,
                         '/dev/hello-motor-lift': False}
        for d in robot_devices:
            try:
                ser = serial.Serial(d)
                if ser.isOpen():
                    try:
                        fcntl.flock(ser.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
                    except IOError:
                        robot_devices[d]=True
            except OSError:
                pass

        self.test.log_data('robot_devices_on_bus',robot_devices)

        for d in robot_devices:
            print('Device %s on bus: %d' %(d,robot_devices[d]))
            self.assertFalse(robot_devices[d], msg='Port %s is busy. Another Stretch Body process is already running' % d)


test_suite = TestSuite(test=Test_SIMPLE_stretch_body_background.test,failfast=False)
test_suite.addTest(Test_SIMPLE_stretch_body_background('test_stretch_body_background'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import val_in_range
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.pimu
import stretch_body.hello_utils as hu
import time
import click
import stretch_body.scope as scope
import os
import signal
import subprocess
from stretch_diagnostics.test_helpers import Scope_Log_Sensor
class test_POWER_battery_loading(unittest.TestCase):
    """
    Test if charger is working
    """
    test = TestBase('test_POWER_battery_loading')

    def start_stress(self):
        print('Starting stress')
        subprocess.Popen("stress -c 4", shell=True)

    def kill_stress(self):
        print('Killing stress')
        for line in os.popen("ps ax | grep stress | grep -v grep"):
            fields = line.split()
            pid = fields[0]
            os.kill(int(pid), signal.SIGKILL)


    def test_battery_loading(self):
        """
        Check if supply mode voltage is OK
        """
        data={}
        p=stretch_body.pimu.Pimu()
        self.assertTrue(p.startup(),'Failed to startup Pimu')
        p.pull_status()

        print()
        click.secho('Unplug charger from robot.', fg="yellow")
        click.secho('This test will take ~60s.', fg="yellow")
        click.secho('Hit enter when ready', fg="yellow")
        input()

        #Measure no load voltage
        image_fn = self.test.test_result_dir + '/battery_no_load_voltage_%s.png' % self.test.timestamp
        sls=Scope_Log_Sensor(duration=10.0,y_range=[9,14.0],title='No charger, no load voltage (V)',num_points=100, image_fn=image_fn, start_fn=None, start_fn_ts=None,end_fn=None, end_fn_ts=None)
        while sls.step(p.status['voltage']):
            p.pull_status()
        data['voltage_no_load'] = sls.avg
        data['voltage_no_load_data']=sls.data

        v_min=10.5

        low_voltage=sls.avg<v_min
        msg='Average no load voltage of %f below expected of %f. Batteries may be dischaged or damaged.'%(sls.avg,v_min)
        self.assertFalse(low_voltage,msg=msg)

        #Measure loaded voltage
        image_fn = self.test.test_result_dir + '/battery_stress_load_voltage_%s.png' % self.test.timestamp
        sls = Scope_Log_Sensor(duration=60.0, y_range=[9, 14.0], title='No charger, stress load voltage (V)', num_points=300,
                               image_fn=image_fn, start_fn=self.start_stress, start_fn_ts=2.0,end_fn=self.kill_stress,end_fn_ts=55.0)
        while sls.step(p.status['voltage']):
            p.pull_status()
        data['voltage_stress_load'] = sls.avg
        data['voltage_stress_load_data']=sls.data

        low_voltage = sls.avg < v_min
        msg = 'Average loaded voltage of %f below expected of %f. Batteries may be dischaged or damaged.' % (sls.avg, v_min)
        self.assertFalse(low_voltage, msg=msg)

        self.test.log_data('test_battery_loading', data)

        p.stop()


test_suite = TestSuite(test=test_POWER_battery_loading.test,failfast=False)
test_suite.addTest(test_POWER_battery_loading('test_battery_loading'))


if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
import unittest
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_suite import TestSuite
from stretch_diagnostics.test_runner import TestRunner
import os


class Test_SIMPLE_filesystem(unittest.TestCase):
    """
    Verify the filesystem integrity of the stretch robot
    """

    test = TestBase('test_SIMPLE_filesystem')

    def check_dir_paths(self, file_dir_list):
        for f in file_dir_list:
            check = os.path.isdir(os.path.expanduser(f))
            self.assertTrue(check, "Directory not found : {}".format(f))

    def check_file_paths(self, file_path_list):
        for f in file_path_list:
            check = os.path.isfile(os.path.expanduser(f))
            self.assertTrue(check, "File not found : {}".format(f))

    def test_check_stretch_user_files(self):
        """
        Verify existance of user files
        """
        files = ["~/stretch_user/{}/stretch_configuration_params.yaml".format(self.test.fleet_id),
                 "~/stretch_user/{}/stretch_user_params.yaml".format(self.test.fleet_id)]
        # ADD More

        dirs = ["~/stretch_user",
                "~/stretch_user/{}/calibration_base_imu".format(self.test.fleet_id),
                "~/stretch_user/{}/calibration_steppers".format(self.test.fleet_id),
                "~/stretch_user/{}/calibration_D435i".format(self.test.fleet_id),
                "~/stretch_user/{}/calibration_guarded_contact".format(self.test.fleet_id),
                "~/stretch_user/{}/calibration_ros".format(self.test.fleet_id),
                "~/stretch_user/{}/calibration_guarded_contact".format(self.test.fleet_id),
                "~/stretch_user/{}/exported_urdf".format(self.test.fleet_id),
                "~/stretch_user/{}/udev".format(self.test.fleet_id)]

        self.test.log_params("stretch_user_dir_checks", dirs)
        self.test.log_params("stretch_user_files_checks", files)
        self.check_dir_paths(dirs)
        self.check_file_paths(files)

    def test_udev_files(self):
        """
        verify if all the udev files copied during stretch install is present
        """
        files = ["~/stretch_user/{}/udev/60-openocd.rules".format(self.test.fleet_id),
                 "~/stretch_user/{}/udev/90-hello-respeaker.rules".format(self.test.fleet_id),
                 "~/stretch_user/{}/udev/91-hello-lrf.rules".format(self.test.fleet_id),
                 "~/stretch_user/{}/udev/94-hello-usb.rules".format(self.test.fleet_id),
                 "~/stretch_user/{}/udev/95-hello-arduino.rules".format(self.test.fleet_id),
                 "~/stretch_user/{}/udev/97-intel-ncs2.rules".format(self.test.fleet_id),
                 "~/stretch_user/{}/udev/99-hello-dynamixel.rules".format(self.test.fleet_id)]
        self.check_file_paths(files)

    def test_autostart_files(self):
        """
        Verify if all autostart scripts present
        """
        files = ["/bin/hello_robot_audio.sh",
                 "/bin/hello_robot_lrf_off.py",
                 "/bin/hello_robot_pimu_ping.py",
                 "/bin/hello_robot_pimu_ping.sh",
                 "/bin/hello_robot_xbox_teleop.sh"]
        self.check_file_paths(files)

    def test_etc_files(self):
        """
        Check if /etc directory has stretch user files and udev
        """
        dirs = ["/etc/hello-robot/{}".format(self.test.fleet_id)]
        files = ["/etc/udev/rules.d/60-openocd.rules",
                 "/etc/udev/rules.d/90-hello-respeaker.rules",
                 "/etc/udev/rules.d/91-hello-lrf.rules",
                 "/etc/udev/rules.d/94-hello-usb.rules",
                 "/etc/udev/rules.d/95-hello-arduino.rules",
                 "/etc/udev/rules.d/97-intel-ncs2.rules",
                 "/etc/udev/rules.d/99-hello-dynamixel.rules"]
        self.check_dir_paths(dirs)
        self.check_file_paths(files)

    def test_storage_space(self):
        """
        TODO
        """
        pass


test_suite = TestSuite(test=Test_SIMPLE_filesystem.test, failfast=False)
test_suite.addTest(Test_SIMPLE_filesystem('test_check_stretch_user_files'))
test_suite.addTest(Test_SIMPLE_filesystem('test_udev_files'))
test_suite.addTest(Test_SIMPLE_filesystem('test_autostart_files'))
test_suite.addTest(Test_SIMPLE_filesystem('test_etc_files'))
# test_suite.addTest(Test_SIMPLE_filesystem('test_storage_space'))

if __name__ == '__main__':
    runner = TestRunner(suite=test_suite, doc_verify_fail=False)
    runner.run()



#!/usr/bin/env python3
import os.path
import sys
import argparse
import stretch_body.hello_utils as hu
from stretch_diagnostics.test_manager import TestManager
from stretch_diagnostics.test_order import test_order
from stretch_diagnostics.test_helpers import extract_zip, center_string, run_gist
import click
from colorama import Style

hu.print_stretch_re_use()

parser = argparse.ArgumentParser(description='Script to run Diagnostic Test Suite and generate reports.', )
parser.add_argument("--report", help="Report the latest diagnostic check", action="store_true")
parser.add_argument("--zip", help="Generate zip file of latest diagnostic check", action="store_true")
parser.add_argument("--archive", help="Archive old diagnostic test data", action="store_true")
parser.add_argument("--menu", help="Run tests from command line menu", action="store_true")
parser.add_argument("--unzip", type=str, metavar='zip file', nargs='?',
                    help="Unzip the given stretch diagnostics zipped data and view report.")
parser.add_argument("--list", type=int, metavar='verbosity', choices=[1, 2], nargs='?',
                    help="Lists all the available TestSuites and its included TestCases Ordered (Default verbosity=1)",
                    const=1)
parser.add_argument("--gist", type=str, metavar='gist ID', nargs='?',
                    help="Run a MISC test from git gist content e.g, --gist username/b85faad0d933cd1cbe98133bfaf37782")

group = parser.add_mutually_exclusive_group()
group.add_argument("--simple", help="Run simple diagnostics across entire robot", action="store_true")
group.add_argument("--power", help="Run diagnostics on the power subsystem", action="store_true")
group.add_argument("--realsense", help="Run diagnostics on the Intel RealSense D435 camera", action="store_true")
group.add_argument("--stepper", help="Run diagnostics on stepper drivers", action="store_true")
group.add_argument("--firmware", help="Run diagnostics on robot firmware versions", action="store_true")
group.add_argument("--dynamixel", help="Run diagnostics on all robot Dynamixel servos", action="store_true")
group.add_argument("--gripper", help="Run diagnostics on the gripper subsystem", action="store_true")
group.add_argument("--ros", help="Run diagnostics on the ROS packages", action="store_true")
group.add_argument("--cpu", help="Run diagnostics on the CPU", action="store_true")
group.add_argument("--arm", help="Run diagnostics on the Arm", action="store_true")
group.add_argument("--lift", help="Run diagnostics on the Lift", action="store_true")
group.add_argument("--all", help="Run all diagnostics", action="store_true")

args = parser.parse_args()


def print_report(suite_names=None):
    print(Style.BRIGHT + center_string('SUMMARY', ch='#') + '\n')
    if suite_names is None:
        suite_names = test_order.keys()
    for t in suite_names:
        print(Style.BRIGHT + center_string(f"{t.upper()} TESTS", ch='#') + Style.RESET_ALL)
        system_check = TestManager(test_type=t)
        system_check.print_status_report(show_subtests=True)
        print('')


def run_test_type(test_type):
    mgmt = TestManager(test_type=test_type)
    if args.menu:
        mgmt.run_menu(show_subtests=False)
    else:
        mgmt.run_suite()


def unzip_print_test_status(zip_file):
    s = zip_file.split('.')
    if not os.path.exists(zip_file) or s[-1]!='zip':
        print('Invalid zip filename')
        return
    dir_name = s[-2].strip('/')
    id = zip_file.find("stretch-re")
    stretch_id = zip_file[id:id + 16]
    if not os.path.isdir(dir_name):
        os.system("mkdir {}".format(dir_name))
    if os.path.isdir(dir_name):
        print("Extracting {}...\n".format(zip_file))
        extract_zip(zip_file, dir_name)
    txt = "Diagnostics Tests Status"
    print("=" * len(txt))
    print(click.style("Robot: {}".format(stretch_id), bold=True))
    print(click.style(txt, bold=True))
    print("=" * len(txt))
    for test_type in test_order:
        tm = TestManager(test_type)
        tm.results_directory = dir_name
        print("\n")
        tx = "{} Tests Status".format(test_type)
        print(click.style(tx, bold=True))
        print("-" * len(tx))
        tm.print_status_report(show_subtests=True)


if args.menu and len(sys.argv) < 3:
    print("The '--menu' tag must be provided with a test type. E.g. stretch_diagnostics_check.py --menu --simple")
    exit()

if args.menu and args.all:
    print("The `--menu` tag cannot be used with `--all` test type.")
    exit()

if (args.list and len(sys.argv) < 3) or (args.list and sys.argv[-1] == 'list'):
    print("The '--list' tag must be prefixed by a test type (E.g. stretch_diagnostics_check.py --simple --list)")
    exit()

if args.archive:
    for t in test_order.keys():
        mgmt = TestManager(test_type=t)
        mgmt.archive_all_tests_status()
    print('Archvied all diagnostic data under: %s' % mgmt.results_directory)

if args.zip:
    print(
        Style.BRIGHT + center_string('Zipping Latest Results', 150, '#') + Style.RESET_ALL)
    zip_file = None
    for t in test_order.keys():
        mgmt = TestManager(test_type=t)
        mgmt.generate_last_diagnostic_report(silent=True)
        zip_file = mgmt.generate_latest_zip_file(zip_file=zip_file)
    print('\n----------- Complete -------------')
    print('Zip file available at: %s' % zip_file)

if args.report and len(sys.argv) > 2:
    print("The '--report' can only be used alone.")
    exit()

if args.report:
    print_report()

if args.unzip:
    fn = str(args.unzip)
    unzip_print_test_status(zip_file=fn)

if args.gist:
    gist_id = str(args.gist)
    run_gist(gist_id)


if args.all:
    for t in test_order.keys():
        if args.list:
            print("\n")
            mgmt = TestManager(test_type=t)
            mgmt.list_ordered_tests(verbosity=int(args.list))
        else:
            run_test_type(t)

if args.simple:
    if args.list:
        mgmt = TestManager(test_type='simple')
        mgmt.list_ordered_tests(verbosity=int(args.list))
    else:
        run_test_type('simple')

if args.power:
    if args.list:
        mgmt = TestManager(test_type='power')
        mgmt.list_ordered_tests(verbosity=int(args.list))
    else:
        run_test_type('power')

if args.dynamixel:
    if args.list:
        mgmt = TestManager(test_type='dynamixel')
        mgmt.list_ordered_tests(verbosity=int(args.list))
    else:
        run_test_type('dynamixel')

if args.gripper:
    if args.list:
        mgmt = TestManager(test_type='gripper')
        mgmt.list_ordered_tests(verbosity=int(args.list))
    else:
        run_test_type('gripper')

if args.stepper:
    if args.list:
        mgmt = TestManager(test_type='stepper')
        mgmt.list_ordered_tests(verbosity=int(args.list))
    else:
        run_test_type('stepper')

if args.realsense:
    if args.list:
        mgmt = TestManager(test_type='realsense')
        mgmt.list_ordered_tests(verbosity=int(args.list))
    else:
        run_test_type('realsense')

if args.ros:
    if args.list:
        mgmt = TestManager(test_type='ros')
        mgmt.list_ordered_tests(verbosity=int(args.list))
    else:
        run_test_type('ros')

if args.arm:
    if args.list:
        mgmt = TestManager(test_type='arm')
        mgmt.list_ordered_tests(verbosity=int(args.list))
    else:
        run_test_type('arm')

if args.lift:
    if args.list:
        mgmt = TestManager(test_type='lift')
        mgmt.list_ordered_tests(verbosity=int(args.list))
    else:
        run_test_type('lift')

if args.cpu:
    if args.list:
        mgmt = TestManager(test_type='cpu')
        mgmt.list_ordered_tests(verbosity=int(args.list))
    else:
        run_test_type('cpu')

if args.gripper:
    if args.list:
        mgmt = TestManager(test_type='gripper')
        mgmt.list_ordered_tests(verbosity=int(args.list))
    else:
        run_test_type('gripper')

if not len(sys.argv) > 1:
    parser.error('No action requested. Please use one of the arguments listed above.')



#!/usr/bin/env python3
import unittest
import subprocess
import os
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_suite import TestSuite
from stretch_diagnostics.test_runner import TestRunner


class Test_ROS_sourced_distro(unittest.TestCase):
    """
    Test to check if correct ROS distro and workspace are sourced in the .bashrc file.
    """

    test = TestBase('test_ROS_sourced_distro')

    def test_ros_install(self):
        """
        Check whether ROS is installed
        """
        
        try:
            distro = subprocess.check_output(["ls", "/opt/ros"])
            distro = str(distro, 'UTF-8')
            distros = distro.split()
            if not distros: # if /opt/ros directory exists but is empty
                self.test.add_hint('No ROS installation detected. Install ROS.')
        except subprocess.CalledProcessError: # if /opt/ros directory does not exist
            distros = []
            self.assertTrue(0,'No ROS installation detected. Install ROS.')

        self.test.log_data('ros_distro_installed', distros)
        self.assertTrue(distros,'No ROS distro installed')

    def test_distro_sourced(self):
        """
        Check whether a ROS distro is sourced
        """

        distro = os.getenv('ROS_DISTRO')

        self.assertTrue(distro,'No ROS distro sourced. Source ROS in the ~/.bashrc file.')

        self.test.log_data('ros_distro_sourced', distro)
        self.assertTrue(distro, 'No ROS distro sourced')
        
    def test_workspace_sourced(self):
        """
        Check whether correct workspace is sourced in .bashrc file
        """

        sourced = False
        ros_ws_sourced = None
        distro = os.getenv('ROS_DISTRO')

        home_path = os.getenv('HOME')
        bashrc_path = '{}/.bashrc'.format(home_path)
        ament_ws_path = '{}/ament_ws/install/setup.bash'.format(home_path)
        catkin_ws_path = '{}/catkin_ws/devel/setup.bash'.format(home_path)

        with open(bashrc_path, 'r') as file:
            line_list = []
            for line in file:
                if 'source' in line:
                    if '#' not in line:
                        line_list.append(line)

        if not line_list:
            self.assertTrue(0,'There is no ros workspace sourced. Source {} workspace in .bashrc file.'.format(distro))
        else:
            for line in line_list:
                if distro in ['melodic', 'noetic'] and catkin_ws_path in line:
                    sourced = True
                    ros_ws_sourced = catkin_ws_path
                elif distro in ['galactic', 'humble'] and ament_ws_path in line:
                    sourced = True
                    ros_ws_sourced = ament_ws_path
                elif distro in ['galactic', 'humble'] and catkin_ws_path in line: # Conflicting workspace
                    sourced = False
                    ros_ws_sourced = catkin_ws_path
                    break
                elif distro in ['melodic', 'noetic'] and ament_ws_path in line: # Conflicting workspace
                    sourced = False
                    ros_ws_sourced = ament_ws_path
                    break

        self.assertTrue(sourced,'Either no or conflicting ros workspace sourced. Source correct ros workspace in .bashrc file.')
        self.test.log_data('ros_ws_sourced', ros_ws_sourced)



# failsafe set to True - if a test fails, subsequent tests will be assumed to fail and won't be run
test_suite = TestSuite(test=Test_ROS_sourced_distro.test, failfast=True)

# Add tests from the Test Class to the test_suite in the same order it would be run.
test_suite.addTest(Test_ROS_sourced_distro('test_ros_install'))
test_suite.addTest(Test_ROS_sourced_distro('test_distro_sourced'))
test_suite.addTest(Test_ROS_sourced_distro('test_workspace_sourced'))

if __name__ == '__main__':
    runner = TestRunner(suite=test_suite, doc_verify_fail=False)
    runner.run()



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import val_in_range
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.pimu
import stretch_body.hello_utils as hu

class Test_SIMPLE_pimu(unittest.TestCase):
    """
    Test USB Devices on Bus
    """
    test = TestBase('test_SIMPLE_pimu')


    def test_pimu_present(self):
        """
        Check that Pimu device is present
        """
        self.assertTrue(hdu.is_device_present('/dev/hello-pimu'),msg='Device /dev/hello-pimu not found')
        ttyACMx=hdu.get_all_ttyACMx()
        self.test.log_data('ttyACMx_devices', ttyACMx)
        p=stretch_body.pimu.Pimu()
        self.assertTrue(p.startup(),msg='Device /dev/hello-pimu found but could not startup')
        p.stop()

    def test_pimu_sensors(self):
        """
        Check Pimu sensors have reasonable values
        """
        p = stretch_body.pimu.Pimu()
        self.assertTrue(p.startup(),'Unable to startup Pimu')
        self.assertTrue(val_in_range('Voltage', p.status['voltage'], vmin=p.config['low_voltage_alert'], vmax=14.5),msg='Voltage of %f outside of bounds'%p.status['voltage'])
        self.assertTrue(val_in_range('Current', p.status['current'], vmin=0.5, vmax=p.config['high_current_alert']),msg='Coltage of %f outside of bounds'%p.status['current'])
        self.assertTrue(val_in_range('Temperature', p.status['temp'], vmin=10, vmax=40),msg='Temp of %f outside of bounds'%p.status['temp'])
        self.assertTrue(val_in_range('Cliff-0', p.status['cliff_range'][0], vmin=p.config['cliff_thresh'], vmax=100),msg='Cliff-0 of %f outside of bounds'%p.status['cliff_range'][0])
        self.assertTrue(val_in_range('Cliff-1', p.status['cliff_range'][1], vmin=p.config['cliff_thresh'], vmax=100),msg='Cliff-1 of %f outside of bounds'%p.status['cliff_range'][1])
        self.assertTrue(val_in_range('Cliff-2', p.status['cliff_range'][2], vmin=p.config['cliff_thresh'], vmax=100),msg='Cliff-2 of %f outside of bounds'%p.status['cliff_range'][2])
        self.assertTrue(val_in_range('Cliff-3', p.status['cliff_range'][3], vmin=p.config['cliff_thresh'], vmax=100),msg='Cliff-3 of %f outside of bounds'%p.status['cliff_range'][3])
        self.assertTrue(val_in_range('IMU AZ', p.status['imu']['az'], vmin=-10.1, vmax=-9.5),msg='IMU AZ of %f outside of bounds'%p.status['imu']['az'])
        self.assertTrue(val_in_range('IMU Pitch', hu.rad_to_deg(p.status['imu']['pitch']), vmin=-12, vmax=12),msg='IMU Pitch of %f outside of bounds'%p.status['imu']['pitch'])
        self.assertTrue(val_in_range('IMU Roll', hu.rad_to_deg(p.status['imu']['roll']), vmin=-12, vmax=12),msg='IMU Roll of %f outside of bounds'%p.status['imu']['roll'])
        p.stop()

test_suite = TestSuite(test=Test_SIMPLE_pimu.test,failfast=False)
test_suite.addTest(Test_SIMPLE_pimu('test_pimu_present'))
test_suite.addTest(Test_SIMPLE_pimu('test_pimu_sensors'))

if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



#!/usr/bin/env python3
import stretch_diagnostics.test_helpers as test_helpers
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.stepper
import stretch_body.hello_utils as hu
import os
import  click
import time
import stretch_body.pimu as pimu

class Test_STEPPER_runstop(unittest.TestCase):
    """
    Test Stepper calibration data consistency
    """
    test = TestBase('test_STEPPER_runstop')

    def stepper_runstop(self,joint,xd,error_thresh):
        """
        Check that stepper motion halts on runstop
        """
        p=pimu.Pimu()
        self.assertTrue(p.startup())
        log={'runstop_off_x0':0,'runstop_off_x1':0,'runstop_on_x0':0,'runstop_on_x1':0}
        print('Testing runstop on stepper %s'%joint)
        motor=stretch_body.stepper.Stepper('/dev/'+joint)
        self.assertTrue(motor.startup())
        motor.disable_sync_mode()
        motor.push_command()

        motor.pull_status()
        log['runstop_off_x0']=motor.status['pos']
        print('Reseting runstop and checking for small motion...')

        p.runstop_event_reset()
        p.push_command()

        motor.set_command(motor.MODE_POS_TRAJ_INCR,x_des=xd)
        motor.push_command()
        time.sleep(1.5)
        motor.pull_status()
        log['runstop_off_x1'] = motor.status['pos']
        self.test.log_data('test_runstop_'+joint, log)
        error=abs(abs(log['runstop_off_x1']-log['runstop_off_x0'])-xd)
        msg='%s: Runstop off and motion of %f (rad) relative to expected of %f'%(joint,abs(log['runstop_off_x1']-log['runstop_off_x0']),xd)
        print(msg)
        self.assertTrue(error<error_thresh,msg=msg) #Within error_thresh radians

        print('Triggering runstop and checking for no motion...')
        p.runstop_event_trigger()
        p.push_command()
        time.sleep(0.5)
        motor.pull_status()
        log['runstop_on_x0']=motor.status['pos']
        motor.set_command(motor.MODE_POS_TRAJ_INCR, x_des=xd)
        motor.push_command()
        time.sleep(1.5)
        motor.pull_status()
        log['runstop_on_x1'] = motor.status['pos']
        self.test.log_data('test_runstop_' + joint, log)
        error = abs(log['runstop_on_x1'] - log['runstop_on_x0'])
        msg = '%s: Runstop on and motion of %f (rad) relative to expected of %f' % (joint,error, 0)
        print(msg)
        self.assertTrue(error < error_thresh, msg=msg)  # Within error_thresh radians
        p.runstop_event_reset()
        p.push_command()
        motor.stop()

    def test_runstop_right_wheel(self):
        "Check that runstop is working for the right wheel"
        self.stepper_runstop('hello-motor-right-wheel',xd=1.0,error_thresh=0.2)

    def test_runstop_left_wheel(self):
        "Check that runstop is working for the left wheel"
        self.stepper_runstop('hello-motor-left-wheel',xd=1.0,error_thresh=0.2)

    def test_runstop_arm(self):
        "Check that runstop is working for the arm"
        print()
        click.secho('Ensure arm is not at a hardstop. Hit enter when ready', fg="yellow")
        input()
        self.stepper_runstop('hello-motor-arm',xd=0.5,error_thresh=0.1)

    def test_runstop_lift(self):
        "Check that runstop is working for the lift"
        print()
        click.secho('Ensure lift is not at a hardstop. Hit enter when ready', fg="yellow")
        input()
        self.stepper_runstop('hello-motor-lift',xd=0.5,error_thresh=0.2)

test_suite = TestSuite(test=Test_STEPPER_runstop.test,failfast=False)
test_suite.addTest(Test_STEPPER_runstop('test_runstop_right_wheel'))
test_suite.addTest(Test_STEPPER_runstop('test_runstop_left_wheel'))
test_suite.addTest(Test_STEPPER_runstop('test_runstop_arm'))
test_suite.addTest(Test_STEPPER_runstop('test_runstop_lift'))
if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



import unittest
import os
import stretch_body.hello_utils as hu
import yaml
from colorama import Fore, Style
from stretch_diagnostics.test_result import TestResult


class TestRunner(unittest.TextTestRunner):
    resultclass = TestResult

    def __init__(self, suite, doc_verify_fail=False):
        super(TestRunner, self).__init__()
        self.failfast = suite.failfast
        self.suite = suite
        self.doc_verify_fail = doc_verify_fail

    def _suite_verify_doc_fail(self):
        doc_check_fail = False
        if len(self.suite._tests) == 0:
            print(Fore.YELLOW + '[SYSTEM CHECK ERROR]: A test suite must have at least one test\n' + Style.RESET_ALL)
            doc_check_fail = True

        if len(self.suite._tests):
            class_doc = self.suite._tests[0].__doc__
            if class_doc is None:
                print(
                    Fore.YELLOW + '[SYSTEM CHECK ERROR]: A test case must have a class-level docstring' + Style.RESET_ALL)
                doc_check_fail = True

            for t in self.suite._tests:
                if not t.shortDescription():
                    print(Fore.YELLOW + '[SYSTEM CHECK ERROR]: Short Description not provided for test : {}'.format(
                        t.id()) + Style.RESET_ALL)
                    doc_check_fail = True

        return doc_check_fail

    def run(self):
        if self.doc_verify_fail:
            if self._suite_verify_doc_fail():
                print(Fore.RED + 'Stopping Test Run' + Style.RESET_ALL)
                return
        sub_tests_info = {}
        for t in self.suite._tests:
            if t.shortDescription():
                sub_tests_info[t.id().split('.')[2]] = {'description': t.shortDescription(), 'status': 'PASS'}
            else:
                sub_tests_info[t.id().split('.')[2]] = {'description': None, 'status': 'PASS'}
        result = super(TestRunner, self).run(self.suite)
        self.test_result_filename=None
        if self.suite.test:
            self.suite.test.sub_tests_info = sub_tests_info
            self.test_result_filename=self.suite.test.save_TestResult(result)
        else:
            print(
                Fore.YELLOW + '[WARNING]: Result Data not saved because Test Object was not included with Test Suite.')
            print(
                'Add test object while initializing  "test_suite = TestSuite(Test_XXX_foo.test)" ' + Style.RESET_ALL)
        return result



import unittest
from colorama import Fore, Style

class TestSuite(unittest.TestSuite):
    def __init__(self, test=None, failfast=False):
        super().__init__(self)
        self.include_test(test)
        self.failfast = failfast

    def include_test(self, test):
        self.test = test
        if not self.test:
            print(Fore.YELLOW + 'Creating Test Suite Without Test' + Style.RESET_ALL)



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import val_in_range
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
import stretch_body.pimu
import stretch_body.hello_utils as hu
import time
import click
from stretch_diagnostics.test_helpers import Scope_Log_Sensor
class test_POWER_charger(unittest.TestCase):
    """
    Test if charger is working
    """
    test = TestBase('test_POWER_charger')

    def test_supply_mode_voltage(self):
        """
        Check if supply mode voltage is OK
        """
        data = {}
        p = stretch_body.pimu.Pimu()
        self.assertTrue(p.startup(),'Failed to startup Pimu')
        p.pull_status()

        click.secho('Plug charger into robot and place charger in SUPPLY mode', fg="yellow")
        click.secho('Hit enter when ready', fg="yellow")
        input()
        print('This will take 20s...')
        time.sleep(5.0)

        # Measure no load voltage
        image_fn = self.test.test_result_dir + '/battery_supply_mode_voltage_%s.png' % self.test.timestamp
        sls = Scope_Log_Sensor(duration=15.0, y_range=[9, 14.0], title='Charger supply mode, no load voltage (V)', num_points=100,
                               image_fn=image_fn, start_fn=None, start_fn_ts=None, end_fn=None, end_fn_ts=None)
        while sls.step(p.status['voltage']):
            p.pull_status()

        data['voltage_supply_load'] = sls.avg
        data['voltage_supply_load_data']=sls.data

        supply_mode_voltage_min=13.0

        low_voltage = sls.avg < supply_mode_voltage_min
        msg = 'Average supply mode voltage of %f below expected of %f. May be broken charger or cable.' % (
        sls.avg, supply_mode_voltage_min)
        self.assertFalse(low_voltage, msg=msg)

        self.test.log_data('test_supply_mode_voltage', data)

test_suite = TestSuite(test=test_POWER_charger.test,failfast=False)
test_suite.addTest(test_POWER_charger('test_supply_mode_voltage'))


if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()



import unittest
import warnings
import os
import stretch_body.hello_utils as hu
import yaml
from colorama import Fore, Style


class TestBase():
    def __init__(self, test_name):
        print(Style.BRIGHT + '{}'.format(test_name))
        print('=' * len(test_name) + Style.RESET_ALL)

        self.timestamp = hu.create_time_string()
        self.test_name = test_name
        self.fleet_id = os.environ['HELLO_FLEET_ID']

        results_directory = os.environ['HELLO_FLEET_PATH'] + '/log/diagnostic_check'  # +self.timestamp

        os.system('mkdir -p %s' % results_directory)
        self.results_directory = results_directory
        self.results_directory_test_specific = self.results_directory + '/' + test_name
        self.params_dict = {}
        self.data_dict = {}
        self.test_status = {}
        self.sub_tests_info = {}

        self.result_data_dict = {'params': None,
                                 'test_status': None,
                                 'data': None,
                                 'FAILS': None,
                                 'ERRORS': None}
        self.check_test_results_directories()

    def add_hint(self, hint):
        warnings.warn('Adding hints to tests is deprecated.', DeprecationWarning, stacklevel=1)

    def check_test_results_directories(self):
        # self.update_production_repo()
        if not os.path.isdir(self.results_directory):
            os.system('mkdir -p {}'.format(self.results_directory))
        self.test_result_dir = '{}/{}'.format(self.results_directory, self.test_name)

        if not os.path.isdir(self.results_directory):
            os.system('mkdir -p {}'.format(self.test_result_dir))

        self.test_file_dir = '{}/{}'.format(self.results_directory, self.test_name)
        if not os.path.isdir(self.test_file_dir):
            os.system('mkdir -p {}'.format(self.test_file_dir))

    def save_test_result(self, test_status=None):
        """
        This function forces the practice of unit tests to produce result dict with the three fields
        """
        self.result_data_dict['test_status'] = test_status
        self.result_data_dict['params'] = self.params_dict
        self.result_data_dict['data'] = self.data_dict

        print(yaml.dump(self.result_data_dict['test_status']))
        self.check_test_results_directories()
        test_file_name = self.test_name + '_' + self.timestamp + '.yaml'
        filename = self.test_file_dir + '/' + test_file_name
        with open(filename, 'w') as file:
            documents = yaml.dump(self.result_data_dict, file)
        print('Test data saved to : {}'.format(filename))
        return filename

    def log_params(self, key, value):
        self.params_dict[key] = value

    def log_data(self, key, value):
        self.data_dict[key] = value

    def log_fails(self, failures):
        self.result_data_dict['FAILS'] = self.parse_TestErrors(failures)

    def log_errors(self, errors):
        self.result_data_dict['ERRORS'] = self.parse_TestErrors(errors)

    def parse_TestErrors(self, failures):
        fails = []
        for f in failures:
            test_id = str(f[0].id().split('.')[2])
            out = str(f[1])
            out_lines = out.split('\n')
            fails.append({test_id: out_lines})
            if test_id in self.sub_tests_info.keys():
                self.sub_tests_info[test_id]['status'] = 'FAIL'
        return fails

    def save_TestResult(self, result):
        ok = result.wasSuccessful()
        errors = result.errors
        failures = result.failures
        if ok:
            print(Fore.GREEN)
            print('All Test Cases passed')
            filename=self.save_test_result(test_status={'status': 'SUCCESS',
                                               'errors': len(errors),
                                               'failures': len(failures),
                                               'subtests_status': self.sub_tests_info})
            print(Style.RESET_ALL)
        else:
            print(Fore.RED)
            print('{} errors and {} failures so far'.format(len(errors), len(failures)))
            self.log_errors(errors)
            self.log_fails(failures)
            print(Style.RESET_ALL)
            filename=self.save_test_result(test_status={'status': 'FAIL',
                                               'errors': len(errors),
                                               'failures': len(failures),
                                               'subtests_status': self.sub_tests_info})
        return filename

    def move_misc_file(self, file_key, filename):
        ff = filename.split('.')

        filename_ts = ff[0] + '_' + self.timestamp + '.' + ff[-1]
        os.system('mv {} {}/{}/{}'.format(filename, self.results_directory, self.test_name, filename_ts))
        self.log_data(file_key, filename_ts)



#!/usr/bin/env python3
from stretch_diagnostics.test_helpers import val_in_range
from stretch_diagnostics.test_base import TestBase
from stretch_diagnostics.test_runner import TestRunner
from stretch_diagnostics.test_suite import TestSuite
import unittest
import stretch_factory.hello_device_utils as hdu
from rplidar import *
import glob

class Test_SIMPLE_rplidar(unittest.TestCase):
    """
    Test USB Devices on Bus
    """
    test = TestBase('test_SIMPLE_rplidar')

    def test_rplidar_present(self):
        """
        Check that hello-lrf device is present
        """
        try:
            nd=len(glob.glob('/dev/ttyUSB*'))>=3
            dp = hdu.is_device_present('/dev/hello-lrf')
            self. assertTrue(nd,'Not enough ttyUSB devices found. Check RPLidar cables.')
            self.assertTrue(dp,'/dev/hello-lrf device not found. Check UDEV rules.')
            #Start and stop device to reset connection (see https://github.com/SkoltechRobotics/rplidar/issues/34)
            rplidar = RPLidar('/dev/hello-lrf')
            rplidar.stop_motor()
            rplidar.stop()
            rplidar.disconnect()
        except RPLidarException:
            self.assertTrue(0,msg='RPLidar exception. Check all cables to device.')

    def test_rplidar_speed(self):
        """
        Measure the scan rate (points-per-sec) for 10s
        """
        ts=time.time()
        try:
            rplidar = RPLidar('/dev/hello-lrf')
            old_t = None
            points_per_sec = []
            for i, scan in enumerate(rplidar.iter_scans(scan_type='express')):
                now = time.time()
                if old_t is None:
                    old_t = now
                    continue
                delta = now - old_t
                print('%.2f points-per-sec' % (len(scan)/delta))
                points_per_sec.append(len(scan)/delta)
                old_t = now
                if now-ts>10.0:
                    break

            ah = sum(points_per_sec) / len(points_per_sec)
            print('Mean points-per-sec: %.2f Hz' % (ah))
            self.test.log_data('rplidar_points_per_sec', points_per_sec)
            self.assertTrue(ah>2500.0,'RPLidar scan rate of %f points-per-sec below threshold of 2500. Check USB cables.'%ah)
        except RPLidarException:
            self.assertTrue(0, msg='RPLidar exception. Check all cables to device.')

    def test_rplidar_scan(self):
        """
        Check that rplidar can generate a complete scan
        """
        try:
            rplidar = RPLidar('/dev/hello-lrf')
            for i, scan in enumerate(rplidar.iter_scans()):
                if i > 0:
                    break
            print('%d: Got %d RPLidar scan measurments' % (i, len(scan)))
            rplidar.stop_motor()
            rplidar.stop()
            rplidar.disconnect()
            self.assertTrue(len(scan)>90,'Scan length %d is too short'%len(scan)) #Should be 125+
            self.test.log_data('rplidar_scan', str(scan))
        except RPLidarException:
            self.assertTrue(0,msg='RPLidar exception. Check all cables to device.')
    def test_rplidar_info(self):
        """
        Check rplidar can report health and info
        """
        try:
            rplidar = RPLidar('/dev/hello-lrf')
            info = rplidar.get_info()
            self.assertTrue(len(info)>0,'Invalid info returned')
            info['firmware']=list(info['firmware']) #Tuple makes YAML choke
            info['health']=list(rplidar.get_health())
            rplidar.stop_motor()
            rplidar.stop()
            rplidar.disconnect()
            self.test.log_data('rplidar_info', info)
        except RPLidarException:
            self.assertTrue(0,msg='RPLidar exception. Check all cables to device.')

test_suite = TestSuite(test=Test_SIMPLE_rplidar.test,failfast=True)
test_suite.addTest(Test_SIMPLE_rplidar('test_rplidar_present'))
test_suite.addTest(Test_SIMPLE_rplidar('test_rplidar_info'))
test_suite.addTest(Test_SIMPLE_rplidar('test_rplidar_scan'))
test_suite.addTest(Test_SIMPLE_rplidar('test_rplidar_speed'))
if __name__ == '__main__':
    runner = TestRunner(test_suite)
    runner.run()
